/*
 * @psiStartFrame.java Version 1.1 05/09/2016
 *
 * Copyright (c) 2016 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package psi.gui;

import java.util.Observer;
import java.util.Observable;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.sql.Timestamp;
import java.sql.Date;
import java.text.SimpleDateFormat;
import java.text.ParseException;

import cmn.cmnStruct;			         // Global Common Data Structure

import gui.guiMemoryPanel2;              // Memory Panel

import iqstrat.iqstratHeadersListStruct; // Well Header List Structure
import iqstrat.iqstratHeadersStruct;     // Well Header Data Structure
import iqstrat.io.ReadHeadersXMLFile;    // Read Well Header XML Stream

import iqstrat.iqstratFilesListStruct;   // File Location List Structure
import iqstrat.iqstratFilesStruct;       // File Location Data Structure
import iqstrat.io.ReadFilesXMLFile;      // Read Pressure File Location Data

import seismic.seisStruct;               // Seismic Data Structure

import psi.io.psiCSVFile;                // Pressure Read & Parse CSV File
import psi.gui.psiControlFrame;          // Pressure-Seismic Plot Control Frame
import psi.gui.psiPlotFrame;             // Pressure X-Y Plot Frame

import util.utilWaitAnimation;           // Utility Wait Animation Class
import util.SwingWorker;                 // Utility Swing Worker Class

/** Class psiStartFrame
 *  <p> This Class Starting Panel for this web application
 *
 *  @version 1.1 05/09/2016
 *  @author  John R. Victorine
 */

public class psiStartFrame extends JFrame implements ActionListener, Observer
{
  // Input Variables

  private cmnStruct              stCMN      = null; // Common Data Structure

  // Global Variables

  private Observable             notifier   = null; // Observables Object

  private iqstratHeadersStruct   stWell     = null; // headers Information Data Structure
  private iqstratFilesListStruct stPSIFiles = null; // Pressure Data Files Location
  private seisStruct             stPSI      = null; // Seismic Data Structure with Pressure Data

  private guiMemoryPanel2        pMemory    = new guiMemoryPanel2(); // Memory Panel
  private psiControlFrame        control    = null; // Pressure-Seismic Plot Control Frame
  private psiPlotFrame           plotXY     = null; // Pressure XY Plot Frame

  private static final int _2_MINS  = 0;
  private static final int _5_MINS  = 1;
  private static final int _30_MINS = 2;
  private static final int _60_MINS = 3;

  private int iSample = _30_MINS;
  private int iPlot   = 0;

  private static final int _PROFILE = 0;
  private static final int _XY_Plot = 1;

  private int iDisplay = _PROFILE;

  // Global Frame Widgets

  private static final int    total_rb = 4;
  private static final String srb[] = {
	"Every 2 minutes",
	"Every 5 minutes",
	"Every 1/2 hour ",
	"Every hour     ",
  };

  private JRadioButton rb[]    = null;

  private JRadioButton rbProfile = new JRadioButton();
  private JRadioButton rbXY_Plot = new JRadioButton();

  // -- Frame Button Widgets

  private JButton btnContinue = new JButton(); // Plot seismic data
//  private JButton btnClear    = new JButton(); // Clear all data
  private JButton btnExit     = new JButton(); // Exit this dialog

  // .. Progress Bar

  private int          iCount  = 0;
  private int          iTotal  = 0;
  private JProgressBar pbWait  = null;

  /** Constructor psiStartFrame( cmnStruct stCMN )
   *  <p> This is the Constructor for this class.
   *  @param stCMN = Global IQSTRAT Data Structure
   */

  public psiStartFrame( cmnStruct stCMN )
  {
    try
    {
      this.stCMN = stCMN;

      readWell();
      readPressureFilesLocation();

      jbInit();
      pMemory.setMemory();
      setButtons();

      addWindowListener( new psiStartFrame_WindowListener() );
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Panel for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
    JPanel pnlTop      = new JPanel();
    JPanel pBar        = new JPanel();

    JPanel pnlCenter   = new JPanel();
    JPanel pnlSample   = new JPanel();
    JPanel pnlDisplay  = new JPanel();

    JPanel pnlBase     = new JPanel();

    ButtonGroup group  = new ButtonGroup();
    ButtonGroup groupD = new ButtonGroup();

    TitledBorder titledBorderStatus = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Load Pressure Files - Status:");
    titledBorderStatus.setTitleFont(new java.awt.Font("Dialog", 1, 12));

    TitledBorder titledBorderSample = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Sample the Raw File Data");
    titledBorderSample.setTitleFont(new java.awt.Font("Dialog", 1, 12));

    TitledBorder titledBorderDisplay = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Select Type of Plot to Display");
    titledBorderDisplay.setTitleFont(new java.awt.Font("Dialog", 1, 12));

    // Create an Observable

    notifier = new psiStartFrameObservable();

    // Add this Dialog to the notifier as observer

    notifier.addObserver(this);

    // Build Panel

    this.setTitle( "Select Sampling Rate for Pressure Files" );
    this.getContentPane().setLayout( new BorderLayout() );

    // Top Panel

    pnlTop.setLayout(new BorderLayout());

    // -- Progress Bar Panel

    pBar.setLayout(new BorderLayout());
    pBar.setBorder(titledBorderStatus);

    pbWait = new JProgressBar(0, 100);
    pbWait.setPreferredSize(new Dimension(200, 30));
    pbWait.setValue( getProgress() );
    pbWait.setStringPainted( true );

    pBar.add(pbWait, BorderLayout.CENTER);

    // Center Panel

    pnlCenter.setLayout( new BorderLayout() );

    pnlSample.setLayout( new GridLayout() );
    pnlSample.setBorder(titledBorderSample);

    rb = new JRadioButton[total_rb];

    for (int i=0; i<total_rb; i++)
    {
      rb[i] = new JRadioButton();
      if (i==iSample) { rb[i].setSelected(true); }
      rb[i].setEnabled(true);

      rb[i].setFont(new java.awt.Font("Dialog", 1, 13));
      rb[i].setText(srb[i]);
      rb[i].addActionListener(this);

      pnlSample.add(rb[i], null);
      group.add(rb[i]);
	}

    // Type of Plot to make Panel

    pnlDisplay.setLayout( new GridLayout() );
    pnlDisplay.setBorder(titledBorderDisplay);

    rbProfile.setFont(new java.awt.Font("Dialog", 1, 13));
    rbProfile.setSelected(true);
    rbProfile.setText("Profile Plot");
    rbProfile.addActionListener(this);

    pnlDisplay.add(rbProfile, null);
    groupD.add(rbProfile);

    rbXY_Plot.setFont(new java.awt.Font("Dialog", 1, 13));
    rbXY_Plot.setText("X-Y 2D Plot");
    rbXY_Plot.addActionListener(this);

    pnlDisplay.add(rbXY_Plot, null);
    groupD.add(rbXY_Plot);

    // Base Panel
    // -- Frame Buttons

    btnContinue.setFont(new java.awt.Font("Dialog", 1, 11));
//    btnContinue.setPreferredSize( new Dimension(120, 25) );
    btnContinue.setText("Continue");
    btnContinue.addActionListener(this);

    btnExit.setFont(new java.awt.Font("Dialog", 1, 11));
//    btnExit.setPreferredSize( new Dimension(120, 25) );
    btnExit.setText("Exit");
    btnExit.addActionListener(this);

    // Add Widgets to Panel

    this.getContentPane().add( pnlTop,    BorderLayout.NORTH );
    pnlTop.add(pBar,                      BorderLayout.NORTH);
    pnlTop.add(pMemory,                   BorderLayout.SOUTH);

    this.getContentPane().add( pnlCenter, BorderLayout.CENTER );
    pnlCenter.add(pnlSample,              BorderLayout.CENTER );
    pnlCenter.add(pnlDisplay,             BorderLayout.SOUTH );

    this.getContentPane().add( pnlBase,   BorderLayout.SOUTH );
    pnlBase.add( btnContinue,             null );
    pnlBase.add( btnExit,                 null );

    // Display the Frame

    this.setSize(new Dimension(600, 300));
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    this.setLocation((d.width - this.getSize().width) / 2,
                     (d.height - this.getSize().height) / 2);

    this.setResizable(true);
    this.setVisible(true);
  }

  /** METHOD createImageIcon()
   *  <p> This Method will create the Frame for this class.
   *  @param path = the path to the image
   *  @return image icon
   */

  protected ImageIcon createImageIcon(String path)
  {
    java.net.URL imgURL = getClass().getResource(path);
    ImageIcon    image  = null;

    if (imgURL != null)
    {
      image = new ImageIcon(imgURL);
    }
    else
    {
       System.err.println("Couldn't find file: " + path);
    }

    return (image);
  }

  /* ===================================================================== *
   * --------------------------- READ DATA ------------------------------- *
   * ===================================================================== */

  /** Method readWell()
   * <p> This method will read the Wellington KGS 1-28 well information xml data
   */

  public void readWell()
  {
	iqstratHeadersListStruct st = null;

    ReadHeadersXMLFile pRead = new ReadHeadersXMLFile(iqstrat.io.ReadHeadersXMLFile.URL);
    st                       = pRead.Process(cmn.cmnStruct.GET_1_28);

    if (st != null)
    {
      if (st.iCount > 0)
	    stWell = iqstrat.iqstratHeadersUtility.copy( st.stItem[0] );
	}
  }

  /** Method readPressureFilesLocation()
   *  <p> This method will read the Pressure File information from the KGS DB
   */

  private void readPressureFilesLocation()
  {
    ReadFilesXMLFile pRead = new ReadFilesXMLFile(iqstrat.io.ReadFilesXMLFile.URL);
    stPSIFiles             = pRead.Process(cmn.cmnStruct.PRESSURE_FILES);
  }

  /** Method readPressureData()
   *  <p> This method will read the pressure file data.
   *  @param  sPath = URL Path to the pressure file
   *  @param  sFile = file name of the pressure file
   *  @return st    = seismic pressure data file structure
   */

  private static seisStruct readPressureData(String sPath, String sFile)
  {
	seisStruct st   = null;
	psiCSVFile pCSV = null;

	if ((sPath != null) && (sFile != null))
	{
	  if ((sPath.length() > 0) && (sFile.length() > 0))
	  {
        pCSV  = new psiCSVFile();

        pCSV.Open(util.utilFileIO.URL_FILE, sPath, sFile);       // Open File
        pCSV.get2Rows();                                         // Count Rows
        pCSV.Close();                                            // Close File

        pCSV.Open(util.utilFileIO.URL_FILE, sPath, sFile);       // Open File
        pCSV.getData(1);                                         // Build Data Structure
        st = seismic.seisUtility.copyData( pCSV.getData() );     // Get Data
        pCSV.Close();                                            // Close File
	  }
	}

	return (st);
  }

  /* ===================================================================== *
   * ----------------------------- CLOSE --------------------------------- *
   * ===================================================================== */

  /** METHOD psiStartFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class psiStartFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e) { close(); }
  }

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    // Input Variables

    stCMN = null; // Global IQSTRAT Data Structure

    // Global Variables

    notifier = null; // Observables Object

    if (stWell != null)
      stWell.delete();
    stWell = null;  // headers Information Data Structure

    if (stPSIFiles != null)
      stPSIFiles.delete();
    stPSIFiles  = null;  // Pressure File Location Structure

    if (stPSI != null)
      stPSI.delete();
    stPSI   = null;  // Seismic Data Structure with Pressure Data

//	if (control != null)
//	  control.close();
//	control = null;

    if (plotXY != null)
      plotXY.close();
    plotXY     = null;

    if (pMemory != null)
      pMemory.close();
    pMemory = null;

    // Global Frame Widgets

    if (rb != null)
    {
      for (int i=0; i<total_rb; i++)
      {
	    if (rb[i] != null)
          rb[i] = null;
	  }
	}
    rb = null;

    rbProfile   = null;
    rbXY_Plot   = null;

    btnContinue = null; // Continue
    btnExit     = null; // Exit

    pbWait      = null;

    dispose();
  }

  /** Method setButtons()
   *  <p> This method will set the Profile Plot buttons depending on the
   *      selected data.
   */

  private void setButtons()
  {
    btnContinue.setEnabled(false);
    for (int i=0; i<total_rb; i++)
      rb[i].setEnabled(false);

    if (iPlot == 0)
    {
      for (int i=0; i<total_rb; i++)
        rb[i].setEnabled(true);
	}

    if ((iPlot == 0) && (stPSIFiles != null) && (stWell != null))
    {
      btnContinue.setEnabled(true);
	}
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getData()
   *  <p> This method retrieves the pressure data and stiches the file data
   *      into one structure by resampling the data.
   */

  private void getData()
  {
    String     sFile    = "";
    String     sPath    = "";
	seisStruct st       = null;
	int        iMinutes = 0;
	double     dCorr    = 0.0;

	if (stPSIFiles != null)
	{
	  iTotal = stPSIFiles.iCount;

	  for (int i=0; i<stPSIFiles.iCount; i++)
	  {
		sPath = new String(stPSIFiles.stItem[i].sURL);
		sFile = new String(stPSIFiles.stItem[i].sFilename);
        dCorr = stPSIFiles.stItem[i].dPSICorr;

		st = readPressureData( sPath, sFile );

        switch(iSample)
        {
          case _2_MINS:
            iMinutes = 2;
            break;
          case _5_MINS:
            iMinutes = 5;
            break;
          case _30_MINS:
            iMinutes = 30;
            break;
          case _60_MINS:
            iMinutes = 60;
            break;
	    }

		st = seismic.seisUtility.resample( st, iMinutes, dCorr );

		if (stPSI == null)
		{
		  stPSI = seismic.seisUtility.transfer( st );
		}
		else
		{
		  stPSI = seismic.seisUtility.merge( stPSI, st);
		}

		iCount++;
		if (pbWait != null) { pbWait.setValue( getProgress() ); }
        if (pMemory != null) { pMemory.freeMemory(); }
	  }

	  plot();
	  setButtons();
	}

    if (pMemory != null) pMemory.freeMemory();
  }

  /** Method addData()
   *  <p> This method will retrieve the Well Data from the XML File Information.
   */

  private seisStruct addData(seisStruct st, int iMinutes, double dCorr)
  {
	st = seismic.seisUtility.resample( st, iMinutes, dCorr );

	if (stPSI == null)
	{
	  stPSI = seismic.seisUtility.transfer( st );
	}
	else
	{
	  stPSI = seismic.seisUtility.merge( stPSI, st);
	}

	iCount++;
	if (pbWait != null) { pbWait.setValue( getProgress() ); }

	return (st);
  }

  /** Method getProgress()
   * <p> This method will compute the percent of the Total wells loaded
   * @return iPercent = Percent of memory used
   */

  public int getProgress()
  {
    int iPercent = 0;

    if (iTotal > 0)
      iPercent = (int) ( (100 * iCount) / iTotal );

    return ( iPercent );
  }

  /* ===================================================================== *
   * ---------------------------- GET WAIT ------------------------------- *
   * ===================================================================== */

  /** Method WaitXMLPath()
   * <p> This method will display the Wait Animation for opening and parsing a
   *     digtial LAS File.
   */

  private void WaitGetData()
  {
    java.net.URL url = null;

    // Get the URL Path to display the Wait Animation

    try
    {
      url = new java.net.URL(cmn.cmnStruct.IQSTRAT_BASE + "/");
    }
    catch(Exception ex)
    {
      System.out.println("Wrong Image Directory" + ex);
    }

    // Start the Wait Animation Dialog

    final utilWaitAnimation pDialog = new utilWaitAnimation(url);
    pDialog.startAnimation();

    final SwingWorker worker = new SwingWorker()
    {
      public Object construct()
      {
        getData();

        return "OK";
      }

      //Runs on the event-dispatching thread.

      public void finished() { pDialog.stopAnimation(); }
    };

    // Start the Wait Animation Thread

    worker.start();  //required for SwingWorker
  }

  /* ===================================================================== *
   * -------------------------- PLOT METHOD ------------------------------ *
   * ===================================================================== */

  /** Method plot()
   *  <p> This method will plot the data
   */

  private void plot()
  {
    Timestamp tsStart = null; // Starting Date
    Timestamp tsEnd   = null; // Ending Date
	long      lhours  = (long) 3600000;

    iPlot = 1;

  	if (control != null)
   	  control.close();
   	control = null;

   	if (plotXY != null)
   	  plotXY.close();
   	plotXY = null;

	switch (iDisplay)
	{
	  case _PROFILE:
        tsStart  = new Timestamp(stPSI.tsStart.getTime());
        tsEnd    = new Timestamp(stPSI.tsEnd.getTime());

    	control = new psiControlFrame( stCMN,
    	                               stWell,
    	                               tsStart,
    	                               tsEnd,
    	                               seismic.seisUtility.copyData( stPSI ));
	    break;
	  case _XY_Plot:
    	plotXY = new psiPlotFrame( stWell, seismic.seisUtility.copyData( stPSI ));
	    break;
	}
  }

  /** Method plot()
   *  <p> This method will plot the data
   *

  private void plot()
  {
    Timestamp tsStart = null; // Starting Date
    Timestamp tsEnd   = null; // Ending Date
	long      lhours  = (long) 3600000;

    iPlot   = 1;

	if (control != null)
	  control.close();
	control = null;

    tsStart  = new Timestamp(stPSI.tsStart.getTime());
    tsEnd    = new Timestamp(stPSI.tsEnd.getTime());

	control = new psiControlFrame( stCMN,
	                               stWell,
	                               tsStart,
	                               tsEnd,
	                               seismic.seisUtility.transfer( stPSI ));
  }
*/
  /** Method plot()
   *  <p> This method will plot the data
   *

  private void plot()
  {
	if (plotXY != null)
	  plotXY.close();
	plotXY = null;

	plotXY = new psiPlotFrame( stWell, seismic.seisUtility.transfer( stPSI ));
  }
*/
  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    for (int i=0; i<total_rb; i++)
    {
      if (event.getSource() == rb[i])
      {
        iSample = i;
	  }
    }

    if (event.getSource() == rbProfile)
    {
	  iDisplay = _PROFILE;
	  if (iPlot == 1) { plot(); }
	}

    if (event.getSource() == rbXY_Plot)
    {
	  iDisplay = _XY_Plot;
	  if (iPlot == 1) { plot(); }
	}

    if (event.getSource() == btnContinue)
    {
	  iPlot = 2;
	  WaitGetData();
	}

    setButtons();

    if (event.getSource() == btnExit)     { close(); }    // Exit
  }

  /* ===================================================================== *
   * ------------------------ OBSERVER ACTIONS --------------------------- *
   * ===================================================================== */

  /** Method update()
   * <p> This method will handle the Observable Actions
   * @param obj = The Observable making the call
   * @param arg = The returned object
   */

  public void update(Observable obj, Object arg)
  {
    String sArg = new String( (String) arg);

    if (sArg.equals("close plot"))
    {
      iPlot   = 0;
	  if (control != null)
	    control.close();
	  control = null;
	}

	setButtons();
  }
}

/** CLASS psiStartFrameObservable()
  *  <p> This Class will handle Observable Actions when action occurs on a
  *      frame or panel
  */

class psiStartFrameObservable extends Observable
{
  public void notifyObservers(Object b)
  {
    setChanged();
    super.notifyObservers(b);
  }
}

/*
 *  @version 1.1 05/09/2016
 *  @author  John Victorine
 */

