/*
 * @psiCSVFile.java Version 1.1 05/09/2016
 *
 * Copyright (c) 2016 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package psi.io;

import java.awt.*;
import java.io.*;
import java.lang.*;
import java.util.*;

import java.sql.Timestamp;
import java.sql.Date;
import java.text.SimpleDateFormat;
import java.text.ParseException;

import util.utilFileIO;
import cmn.cmnString;
import seismic.seisStruct; // Seismic Data Structure

/** Class psiCSVFile
 *  <p> This Class is designed to open an ascii text rock data CSV file and read
 *      the contents and parse the data into the rock Structure.
 *
 *  @version 1.1 05/09/2016
 *  @author  John R. Victorine
 */

public class psiCSVFile
{
  private utilFileIO  pFILE  = null;

  private String      sLine1 = "";
  private String      sLine2 = "";
  private seisStruct  stData = null;

  private int         iRows    = 0;
  private int         iColumns = 0;

//  private SimpleDateFormat dfm   = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSSSSS");
//  private SimpleDateFormat dfm    = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
  private SimpleDateFormat dfm    = new SimpleDateFormat("MM-dd-yyyy HH:mm:ss");
  private long             ltime  = (long) 0;
  private int              iTime  = 0;
  private int              iMag   = 0;

  private int         iError   = util.utilFileIO.NOERROR;
  private String      sError   = new String("");

  /** Construct psiCSVFile()
   *  <p> This is the Constructor for this class.
   */

  public psiCSVFile() { }

  /* ====================================================================== *
   * ---------------------------- FILE METHODS ---------------------------- *
   * ====================================================================== */

  /** Method Open()
    * <p> This is the Constructor for this class.
    * @param iFileID    = To access the file as URL or FILE.
    * @param sDirectory = File URL or Directory Path
    * @param sFilename  = File Name
    */

  public void Open(int iFileID, String sDirectory, String sFilename)
  {
    pFILE   = new utilFileIO();
    pFILE.Open(iFileID, util.utilFileIO.READ, sDirectory, sFilename);
    iError = pFILE.getErrorID();
    sError = pFILE.getError();

//    iRows             = 0;
//    iColumns          = 0;
  }

  /** Method Close()
   *  <p> This method closes the File.
   */

  public void Close() { pFILE.Close(); }

  /** Method delete()
   * <p> This method will set all the structures to null to force java to free
   *     the allocated memory
   */

  public void delete()
  {
    sLine1   = null;
    sLine2   = null;
    sError   = null;

    if (pFILE != null)
      pFILE.delete();
    pFILE = null;
  }

  /** Method get2Rows()
   * <p> This method will read and parse the Rock File and get the first 2 lines.
   */

  public void get2Rows()
  {
    int             iCount  = 0;
    String          sTemp   = "";
    StringTokenizer stToken = null;

    if (iError == util.utilFileIO.NOERROR)
    {
      while (!pFILE.Next().equals(util.utilFileIO.EOF))
      {
        iError = pFILE.getErrorID();
        sError = pFILE.getError();

        sTemp = pFILE.getString();
        if (iCount == 0)
        {
          sLine1   = new String(sTemp);
          stToken  = new StringTokenizer(sTemp, ",");
          iColumns = stToken.countTokens();
        }

        if (iCount == 1)
          sLine2 = new String(sTemp);

        iCount++;
        iRows++;
      }
    }
  }

  /** Method getData()
   * <p> This method will read and parse the Rock File into its data structures.
   * @param iStart  = the starting row number
   */

  public void getData(int iStart)
  {
    int        i=0;
    int        iCount = 1;
    int        icnt   = 0;
    String     sTemp  = "";
    int        iValue = 0;
    int        iBegin = 0;

    stData = new seisStruct();

    if (iRows > 0)
    {
      stData.iRows = iRows;                // Total Number of rows
      stData.ts    = new Timestamp[iRows]; // Time Stamp of data record
      stData.data  = new double[iRows];    // Data record
	}

    if (iError == util.utilFileIO.NOERROR)
    {
      while (!pFILE.Next().equals(util.utilFileIO.EOF))
      {
        iError = pFILE.getErrorID();
        sError = pFILE.getError();

        if (iError == util.utilFileIO.NOERROR)
        {
          if (iCount >= iStart)
          {
            sTemp  = pFILE.getString();
            stData = parseRow(icnt, sTemp, stData);
            icnt++;

            sTemp = "";
          }

          iCount++;
        }
      }
    }
  }

  /** Method parseRow()
   * <p> This method will parse a Rock File row into the rock structure.
   * @param  iRow    = Row Number
   * @param  sData   = Data Row String
   * @return st      = the Data Structure
   */

  private seisStruct parseRow(int iRow, String sData, seisStruct st)
  {
    int             iCount    = 0;
    String          sTemp     = "";
    StringTokenizer stToken   = null;
    String          sTokens[] = null;

    if (iRows > 0)
    {
      sTemp    = new String(sData.replace('\t', ' '));

      stToken  = new StringTokenizer(sTemp, ",");
      iColumns = stToken.countTokens();

      for (int i=0; i<iColumns; i++)
      {
        sTemp = stToken.nextToken();
        sTemp = new String(sTemp.trim());

	    if (i==0)
	    {
          try
          {
            ltime   = dfm.parse(sTemp).getTime();
            st.ts[iRow] = new Timestamp( ltime );
   	      }
          catch (ParseException e)
          {
            e.printStackTrace();
          }

          if (iTime == 0)
          {
            st.tsStart = new Timestamp(st.ts[iRow].getTime()); // Starting Date
            st.tsEnd   = new Timestamp(st.ts[iRow].getTime()); // Ending Date
            iTime = 1;
  	      }
	      else
            st.tsEnd   = new Timestamp(st.ts[iRow].getTime()); // Ending Date
	    }

        if (i==2)
        {
  	      st.data[iRow] = 0.0;
          if (cmn.cmnString.isNumeric(sTemp))
          {
            st.data[iRow] = cmn.cmnString.stringToDouble(sTemp);
          }

          if (st.data[iRow] > 0.0)
          {
	  	    if (iMag == 0)
	 	    {
			  if (st.data[iRow] > 0.0)
			  {
	            st.dMinimum = st.data[iRow];
	            st.dMaximum = st.data[iRow];
	            iMag++;
			  }
		    }

            if (iMag > 0)
            {
	          if ((st.dMinimum > st.data[iRow]) && (st.data[iRow] > 0.0))
	            st.dMinimum = st.data[iRow];

	          if (st.dMaximum < st.data[iRow])
	            st.dMaximum = st.data[iRow];
			}
		  }
	    }
      }
    }

    return (st);
  }

  /* ====================================================================== *
   * ----------------------------- GET METHODS ---------------------------- *
   * ====================================================================== */

  /** Method getLine1()
   * <p> This method will return the 1st Line of the delimited file
   * @return sLine1 = 1st line of delimited file
   */

  public String getLine1() { return (sLine1); }

  /** Method getLine2()
   * <p> This method will return the 2nd Line of the delimited file
   * @return sLine2 = 2nd line of delimited file
   */

  public String getLine2() { return (sLine2); }

  /** Method getRock()
   * <p> This method will return rock data list
   * @return stData = rock Data list structure
   */

  public seisStruct getData() { return (stData); }

  /* ====================================================================== *
   * -------------------------- GET ERROR METHODS ------------------------- *
   * ====================================================================== */

  /** METHOD getErrorID()
   *  <p> This method will return the error number.
   *  @return iError = Error Indicator
   */

  public int getErrorID() { return (iError); }

  /** METHOD getError()
   *  <p> This method will return the error string.
   *  @return sError = Error String
   */

  public String getError() { return (sError); }
}

/*
 *  @version 1.1 05/09/2016
 *  @author  John Victorine
 */
