/*
 * @psiMathTides.java Version 1.1 05/12/2016
 *
 * Copyright (c) 2016 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package psi.math;

import java.sql.Timestamp;
import java.sql.Date;
import java.text.SimpleDateFormat;
import java.text.ParseException;
import java.awt.*;

import math.Complex;

import seismic.seisStruct;

/** Class psiMathTides
 *  <p> This Class is math package for computing the Solar & Lunar Tides.
 *
 *  @version 1.0 05/12/2016
 *  @author  John R. Victorine
 */

public class psiMathTides
{
  public static final int    _SUN      = psi.math.psiMathTidesConstants._SUN;
  public static final int    _MOON     = psi.math.psiMathTidesConstants._MOON;

  public static final double Go        = psi.math.psiMathTidesConstants.Go;
  public static final double M         = psi.math.psiMathTidesConstants.kgMoon;
  public static final double D         = psi.math.psiMathTidesConstants.D;
  public static final double a         = psi.math.psiMathTidesConstants.a;
  public static final double g         = psi.math.psiMathTidesConstants.g;
  public static final double GPA       = psi.math.psiMathTidesConstants.GPa;
  public static final double PA_psi    = psi.math.psiMathTidesConstants.Pa_psi;

  public static final double MSun      = psi.math.psiMathTidesConstants.kgSun;
  public static final double DSun      = psi.math.psiMathTidesConstants.DSun;
  public static final double AUSun     = psi.math.psiMathTidesConstants.AUSun;
  public static final int    CWrows    = psi.math.psiMathTidesConstants.CWrows;
  public static final double CW[][]    = psi.math.psiMathTidesConstants.CW;

  public static final double HZ_Earth  = psi.math.psiMathTidesConstants.HZ_Earth;
  public static final double KGS_1_28  = psi.math.psiMathTidesConstants.KGS_1_28;

  public static final int    _RA       = psi.math.psiMathTidesConstants._RA;
  public static final String RASun[][] = psi.math.psiMathTidesConstants.RASun;
  public static final String RA[][]    = psi.math.psiMathTidesConstants.RA;

  /** -----------------------------------------------------------------------------
   *
   *  Total deformation due to an earth tide:  theta = 0.49 * W / (a * g)
   *    where W   = tide-generating potential
   *          a   = radius of the earth
   *          g   = acceleration due to gravity
   *
   *  Increase of Pressure due to the earth tide is:  P = rho*g*dh = theta / (Cw * phi)
   *    where phi = porosity of the aquifer
   *          Cw  = water compressibility
   *
   *  -----------------------------------------------------------------------------
   *  The tide-generating potential W
   *      "As the gravitational force of attraction between two masses is inversely
   *	   proportional to the square of the distance between these two
   *	   masses, the potential derived from this force will be inversely proportional
   *	   to the cube of the distance between the two masses. The tidegenerating
   *	   potential can be defined using a spherical harmonic of the
   *       second order, W (Melchior, 1966):
   *
   *           1   G * M  a   a
   *       W = - * -----  - * - * ( 3 cos(beta) * cos(beta) -1 )
   *           2     D    D   D
   *
   *       cos(beta) = sin(le) sin(lp) + cos(le) cos(lp) * cos(Pe - Pp)
   *
   *       where le = latitude on earth
   *             Pe = longitude on earth = wt; where 2PI/w = Earth rotational Period ~24 Hours
   *             lp = latitude of moon with respect to earth
   *             Pp = longitude of moon with respect to earth
   *                               _
   *           3   G * M  a   a   |  1
   *       W = - * -----  - * - * |  -- * (3 cos(2*le) - 1) * (3 cos(2*lp) - 1) +
   *           4     D    D   D   |  12
   *						       -
   *                                    sin(lp) * sin(le) * cos(wt + Pp) +
   *                                                                             _
   *                                    cos^2(lp) * cos^2(le) * cos[2*(wt + Pp)]  |
   *                                                                             -
   *  -----------------------------------------------------------------------------
   *  REFERENCES:
   *
   *  Response of Well-Aquifer Systems to Earth Tides bu John D. Bredheft
   *    Journal of Geophysical Research Volume 72, Number 12, June 15, 1967
   *    http://onlinelibrary.wiley.com/doi/10.1029/JZ072i012p03075/epdf
   *
   *  The Earth Tide Effects on Petroleum Reservoirs - Preliminary Study
   *    by C. Arditty - May 1978, Standord University
   *    https://es.stanford.edu/ERE/pdf/pereports/Engineer/Arditty78.pdf
   *
   *  THE EARTH TIDES by Petr Vanicek, Department of Surveying Engineering,
   *    University of New Brunswick, P.O. Box 4400, Fredericton, N.B. Canada E3B 5A3
   *
   ** ----------------------------------------------------------------------------- */

  /** Method seisStruct normalize()
   *  <p> This method will copy the seismic data structure
   *  @param  stOld   = Old seismic data structure
   *  @param  stStart = Starting Date & Time of data set
   *  @param  stEnd   = Ending Date & Time of data set
   *  @param  dPStart = Starting Pressure "Mean" Value      [psi]
   *  @param  dPEnd   = Ending Pressure "Mean" Value        [psi]
   *  @return stNew   = New seismic data structure
   */

  public static seisStruct normalize( seisStruct stOld,
                                      Timestamp tsStart, Timestamp tsEnd,
                                      double    dPStart, double    dPEnd )
  {
	seisStruct stNew  = null;
	int        iCount = 0;
	long       lstart = (long) 0;
	long       lend   = (long) 0;
	long       ltime  = (long) 0;
	int        iStart = 0;
	double     dMean  = 0.0;
	double     dTime  = 0.0;
	double     dRange = 0.0;

	if (stOld != null)
	{
	  stNew          = new seisStruct();

      stNew.network  = new String(stOld.network);  // Network Code
      stNew.station  = new String(stOld.station);  // Station Identifier
      stNew.location = new String(stOld.location); // Location Identifier
      stNew.channel  = new String(stOld.channel);  // Channel Identifier

      stNew.sData    = new String(stOld.sData);    // Type of Filter
      stNew.sData2   = new String(stOld.sData2);   // Specifics of Filtering
      stNew.sData3   = new String(stOld.sData3);   // Specifics of Filtering

      stNew.tsStart  = new Timestamp(tsStart.getTime());
      stNew.tsEnd    = new Timestamp(tsEnd.getTime());

      stNew.ts       = new Timestamp[stOld.iRows];    // Time Stamp of data record
      stNew.data     = new double[stOld.iRows];       // Data record

	  if (stOld.iRows > 0)
	  {
        lstart = tsStart.getTime();
        lend   = tsEnd.getTime();

		for (int i=0; i<stOld.iRows; i++)
		{
		  ltime = stOld.ts[i].getTime();

		  if ((ltime >= lstart) && (ltime <= lend))
		  {
		    stNew.tsEnd        = new Timestamp(ltime);

		    stNew.ts[iCount]   = new Timestamp(ltime);

		    dTime  = (double) (lend - ltime) / 1000.0;
		    dRange = (double) (lend - lstart) / 1000.0;
  		    dMean  = dPEnd - (dPEnd-dPStart) * dTime / dRange;

		    stNew.data[iCount] = stOld.data[i] - dMean;

  		    if (iStart == 0)
  		    {
		  	  stNew.dMinimum = stNew.data[iCount];
			  stNew.dMaximum = stNew.data[iCount];
			  iStart = 1;
		    }

		    if (stNew.dMinimum > stNew.data[iCount])
		      stNew.dMinimum = stNew.data[iCount];

		    if (stNew.dMaximum < stNew.data[iCount])
		      stNew.dMaximum = stNew.data[iCount];

            stNew.dMinimum = (double) ((int) (stNew.dMinimum *10000.0) / 10000.0);
            stNew.dMaximum = (double) ((int) (stNew.dMaximum *10000.0) / 10000.0);

		    iCount++;
		  }
		}
	  }
      stNew.iRows = iCount;
   	}

	return (stNew);
  }

  /** Method seisStruct psiRemoveTidal()
   *  <p> This method will remove the solar & lunar tide from the pressure signal
   *  @param  stOld   = Old seismic data structure
   *  --------------------------------------------------
   *  @param  le       = latitude of the well
   *  @param  phi      = porosity of the auquifer           [PU]
   *  @param  Cw       = compressibility of water           1/[GPa]
   *  --------------------------------------------------
   *  @param  iHours   = central time number of hours from UTC
   *  --------------------------------------------------
   *  @param  stStart = Starting Date & Time of data set
   *  @param  stEnd   = Ending Date & Time of data set
   *  --------------------------------------------------
   *  @param  dPHIc    = correction
   *  --------------------------------------------------
   *  @return stNew   = New seismic data structure
   */

  public static seisStruct psiRemoveTidal( seisStruct stOld,
                                           double le, double phi, double Cw, int iHours,
                                           Timestamp  tsStart, Timestamp tsEnd,
                                           double dphiC )
  {
	seisStruct stNew  = null;
	int        iCount = 0;
	long       lstart = (long) 0;
	long       lend   = (long) 0;
	long       ltime0 = (long) 0;
	long       ltime  = (long) 0;
	long       ltime2 = (long) 0;
	int        iStart = 0;
	double     dMean  = 0.0;
	double     dTime  = 0.0;
	double     dPHIt  = 0.0;
	double     dRange = 0.0;
	double     dpsi   = 0.0;
	double     dpsio  = 0.0;

	long             lhours   = (long) 3600000;
    double           latMoon  = 0.0; // latitude of the Moon
    double           HRMoon   = 0.0; // RA time of the Moon [secs]
    double           HRTime   = 0.0; // Pressure time at Well [secs]
    double           distMoon = 0.0; // distance between Earth & Moon [m]

    double           latSun   = 0.0; // latitude of the Sun
    double           HRSun    = 0.0; // RA time of the Sun [secs]
    double           distSun  = 0.0; // distance between Earth & Sun [m]

    SimpleDateFormat dfm      = new SimpleDateFormat("MM-dd-yyyy HH:mm:ss");
    double           dRA[]    = null;
    double           dMoon[]  = null;
    double           dEM[]    = null;
    long             lMoon[]  = null;

    double           dRASun[] = null;
    double           dSun[]   = null;
    double           dES[]    = null;
    long             lSun[]   = null;

    dRA   = new double[_RA];
    dMoon = new double[_RA];
    dEM   = new double[_RA];
    lMoon = new long[_RA];

    dRASun = new double[_RA];
    dSun   = new double[_RA];
    lSun   = new long[_RA];
    dES    = new double[_RA];

    Timestamp ts = null;
    Timestamp ts2 = null;

	if (stOld != null)
	{
      for (int i=0; i<_RA; i++)
      {
        try
        {
          lMoon[i] = dfm.parse( RA[i][0] ).getTime();
          lSun[i]  = dfm.parse( RASun[i][0] ).getTime();
     	}
        catch (ParseException e)
        {
          e.printStackTrace();
        }

  	    dRA[i]   = computeRATimeToDecimal( RA[i][1] );
	    dMoon[i] = le * cmn.cmnString.stringToDouble( RA[i][2] ) / 90.0;
	    dEM[i]   = 1000.0 * cmn.cmnString.stringToDouble( RA[i][3] );

  	    dRASun[i] = computeRATimeToDecimal( RASun[i][1] );
	    dSun[i]   = le * cmn.cmnString.stringToDouble( RASun[i][2] ) / 90.0;
	    dES[i]    = AUSun * cmn.cmnString.stringToDouble( RASun[i][3] );
	  }

	  stNew          = new seisStruct();

      stNew.network  = new String(stOld.network);  // Network Code
      stNew.station  = new String(stOld.station);  // Station Identifier
      stNew.location = new String(stOld.location); // Location Identifier
      stNew.channel  = new String(stOld.channel);  // Channel Identifier

      stNew.sData    = new String(stOld.sData);    // Type of Filter
      stNew.sData2   = new String(stOld.sData2);   // Specifics of Filtering
      stNew.sData3   = new String(stOld.sData3);   // Specifics of Filtering

      stNew.tsStart  = new Timestamp(tsStart.getTime());
      stNew.tsEnd    = new Timestamp(tsEnd.getTime());

      stNew.ts       = new Timestamp[stOld.iRows];    // Time Stamp of data record
      stNew.data     = new double[stOld.iRows];       // Data record

      stNew.dMinimum = stOld.dMinimum;
      stNew.dMaximum = stOld.dMaximum;

      stNew.dMinimum = (double) ((int) (stNew.dMinimum *100.0) / 100.0);
      stNew.dMaximum = (double) ((int) (stNew.dMaximum *100.0) / 100.0);

      lstart         = tsStart.getTime();
      lend           = tsEnd.getTime();

      ltime0         = stOld.ts[0].getTime();
//      ltime0         = getUT0(stOld.ts[0].toString());
System.out.println(stOld.ts[0].toString());
      dPHIt = getTimeAsAngle( stOld.ts[0].toString() )-180.0;
//      dphiC = dphiC + dPHIt;
System.out.println(dphiC+" "+dPHIt);

      for (int i=0; i<stOld.iRows; i++)
	  {
	    ltime = stOld.ts[i].getTime();
	    ltime2 = stOld.ts[i].getTime() + iHours * lhours;

  	    if ((ltime >= lstart) && (ltime <= lend))
		{
		  stNew.ts[iCount]   = new Timestamp(ltime);

          for (int k=0; k<_RA-1; k++)
          {
  		    if ((ltime >= lMoon[k]) && (ltime < lMoon[k+1]))
		    {
			  HRMoon   = computeRA( lMoon[k], lMoon[k+1], ltime, dRA[k], dRA[k+1] );  // [hrs]
			  latMoon  = computeLatitude( lMoon[k], lMoon[k+1], ltime, dMoon[k], dMoon[k+1]);
			  distMoon = computeLatitude( lMoon[k], lMoon[k+1], ltime, dEM[k], dEM[k+1]);
		    }

  		    if ((ltime >= lSun[k]) && (ltime < lSun[k+1]))
		    {
			  HRSun    = computeRA( lSun[k], lSun[k+1], ltime, dRASun[k], dRASun[k+1] );  // [hrs]
			  latSun   = computeLatitude( lSun[k], lSun[k+1], ltime, dSun[k], dSun[k+1]);
			  distSun  = computeLatitude( lSun[k], lSun[k+1], ltime, dES[k], dES[k+1]);
		    }
	      }

//          dpsi  = computeTidalPSI( _MOON, lstart, ltime, HRMoon, distMoon, le, latMoon, phi, Cw, dphiC );  // [psi]
//          dpsio = computeTidalPSI( _SUN,  lstart, ltime, HRSun,  distSun,  le, latSun,  phi, Cw, dphiC );  // [psi]
          dpsi  = computeTidalPSI( _MOON, ltime0, ltime, HRMoon, distMoon, le, latMoon, phi, Cw, dphiC );  // [psi]
          dpsio = computeTidalPSI( _SUN,  ltime0, ltime, HRSun,  distSun,  le, latSun,  phi, Cw, dphiC );  // [psi]

		  stNew.data[iCount] = stOld.data[i] + ( dpsi + dpsio );
//		  stNew.data[iCount] = stOld.data[i] - ( dpsi + dpsio );
		  iCount++;
		}
	  }
	}

    stNew.iRows = iCount;

	return (stNew);
  }

  /** Method seisStruct psiArtificial()
   *  <p> This method will copy the seismic data structure
   *  @param  stOld   = Old seismic data structure
   *  --------------------------------------------------
   *  @param  le       = latitude of the well
   *  @param  phi      = porosity of the auquifer           [PU]
   *  @param  Cw       = compressibility of water           1/[GPa]
   *  --------------------------------------------------
   *  @param  iHours   = central time number of hours from UTC
   *  --------------------------------------------------
   *  @param  stStart = Starting Date & Time of data set
   *  @param  stEnd   = Ending Date & Time of data set
   *  @param  dPStart = Starting Pressure "Mean" Value      [psi]
   *  @param  dPEnd   = Ending Pressure "Mean" Value        [psi]
   *  --------------------------------------------------
   *  @param  dPHIc    = correction
   *  --------------------------------------------------
   *  @return stNew   = New seismic data structure
   */

  public static seisStruct psiArtificial( seisStruct stOld,
                                          double le, double phi, double Cw, int iHours,
                                          Timestamp  tsStart, Timestamp tsEnd,
                                          double     dPStart, double    dPEnd,
                                          double dPHIc )
  {
	seisStruct stNew  = null;
	int        iCount = 0;
	long       lstart = (long) 0;
	long       lend   = (long) 0;
	long       ltime0 = (long) 0;
	long       ltime  = (long) 0;
	long       ltime2 = (long) 0;
	int        iStart = 0;
	double     dMean  = 0.0;
	double     dTime  = 0.0;
	double     dPHIt  = 0.0;
	double     dRange = 0.0;
	double     dpsi   = 0.0;
	double     dpsio  = 0.0;

	long             lhours   = (long) 3600000;
    double           latMoon  = 0.0; // latitude of the Moon
    double           HRMoon   = 0.0; // RA time of the Moon [secs]
    double           distMoon = 0.0; // distance between Earth & Moon [m]

    double           latSun   = 0.0; // latitude of the Sun
    double           HRSun    = 0.0; // RA time of the Sun [secs]
    double           distSun  = 0.0; // distance between Earth & Sun [m]

    SimpleDateFormat dfm      = new SimpleDateFormat("MM-dd-yyyy HH:mm:ss");
    double           dRA[]    = null;
    double           dMoon[]  = null;
    double           dEM[]    = null;
    long             lMoon[]  = null;

    double           dRASun[] = null;
    double           dSun[]   = null;
    double           dES[]    = null;
    long             lSun[]   = null;
    int icnt=0;

    dRA    = new double[_RA];
    dMoon  = new double[_RA];
    lMoon  = new long[_RA];
    dEM    = new double[_RA];

    dRASun = new double[_RA];
    dSun   = new double[_RA];
    lSun   = new long[_RA];
    dES    = new double[_RA];

    Timestamp ts = null;
    Timestamp ts2 = null;

	if (stOld != null)
	{
      for (int i=0; i<_RA; i++)
      {
        try
        {
          lMoon[i] = dfm.parse( RA[i][0] ).getTime();
          lSun[i]  = dfm.parse( RASun[i][0] ).getTime();
     	}
        catch (ParseException e)
        {
          e.printStackTrace();
        }

  	    dRA[i]    = computeRATimeToDecimal( RA[i][1] );
	    dMoon[i]  = le * cmn.cmnString.stringToDouble( RA[i][2] ) / 90.0;
	    dEM[i]    = 1000.0 * cmn.cmnString.stringToDouble( RA[i][3] );
//System.out.println(i+" "+dRA[i]+" "+dMoon[i]+" "+dEM[i]+" "+(new Timestamp(lMoon[i])).toString());

  	    dRASun[i] = computeRATimeToDecimal( RASun[i][1] );
	    dSun[i]   = le * cmn.cmnString.stringToDouble( RASun[i][2] ) / 90.0;
	    dES[i]    = AUSun * cmn.cmnString.stringToDouble( RASun[i][3] );
	  }

	  stNew          = new seisStruct();

      stNew.network  = new String(stOld.network);  // Network Code
      stNew.station  = new String(stOld.station);  // Station Identifier
      stNew.location = new String(stOld.location); // Location Identifier
      stNew.channel  = new String(stOld.channel);  // Channel Identifier

      stNew.sData    = new String(stOld.sData);    // Type of Filter
      stNew.sData2   = new String(stOld.sData2);   // Specifics of Filtering
      stNew.sData3   = new String(stOld.sData3);   // Specifics of Filtering

      stNew.tsStart  = new Timestamp(tsStart.getTime());
      stNew.tsEnd    = new Timestamp(tsEnd.getTime());

      stNew.ts       = new Timestamp[stOld.iRows];    // Time Stamp of data record
      stNew.data     = new double[stOld.iRows];       // Data record

      stNew.dMinimum = stOld.dMinimum;
      stNew.dMaximum = stOld.dMaximum;

      stNew.dMinimum = (double) ((int) (stNew.dMinimum *100.0) / 100.0);
      stNew.dMaximum = (double) ((int) (stNew.dMaximum *100.0) / 100.0);

      lstart         = tsStart.getTime();
      lend           = tsEnd.getTime();

      ltime0         = stOld.ts[0].getTime();
//      ltime0 = getUT0(stOld.ts[0].toString());

//      dPHIt = getTimeAsAngle( stOld.ts[0].toString() );
//      dPHIc = dPHIc + dPHIt;

      for (int i=0; i<stOld.iRows; i++)
	  {
	    ltime = stOld.ts[i].getTime();
	    ltime2 = stOld.ts[i].getTime() + iHours * lhours;

  	    if ((ltime >= lstart) && (ltime <= lend))
		{
		  stNew.ts[iCount]   = new Timestamp(ltime);

		  dTime  = (double) (lend - ltime) / 1000.0;
		  dRange = (double) (lend - lstart) / 1000.0;
		  dMean  = dPEnd - (dPEnd-dPStart) * dTime / dRange;

          for (int k=0; k<_RA-1; k++)
          {
  		    if ((ltime >= lMoon[k]) && (ltime < lMoon[k+1]))
		    {
			  HRMoon   = computeRA( lMoon[k], lMoon[k+1], ltime, dRA[k], dRA[k+1] );  // [hrs]
			  latMoon  = computeLatitude( lMoon[k], lMoon[k+1], ltime, dMoon[k], dMoon[k+1]);
			  distMoon = computeLatitude( lMoon[k], lMoon[k+1], ltime, dEM[k], dEM[k+1]);
		    }

  		    if ((ltime >= lSun[k]) && (ltime < lSun[k+1]))
		    {
			  HRSun    = computeRA( lSun[k], lSun[k+1], ltime, dRASun[k], dRASun[k+1] );  // [hrs]
			  latSun   = computeLatitude( lSun[k], lSun[k+1], ltime, dSun[k], dSun[k+1]);
			  distSun  = computeLatitude( lSun[k], lSun[k+1], ltime, dES[k], dES[k+1]);
		    }
	      }

//          dpsi  = computeTidalPSI( _MOON, lstart, ltime, HRMoon, distMoon, le, latMoon, phi, Cw, dPHIc );  // [psi]
//          dpsio = computeTidalPSI( _SUN,  lstart, ltime, HRSun,  distSun,  le, latSun,  phi, Cw, dPHIc );  // [psi]
          dpsi  = computeTidalPSI( _MOON, ltime0, ltime, HRMoon, distMoon, le, latMoon, phi, Cw, dPHIc );  // [psi]
          dpsio = computeTidalPSI( _SUN,  ltime0, ltime, HRSun,  distSun,  le, latSun,  phi, Cw, dPHIc );  // [psi]

		  stNew.data[iCount] = dMean - ( dpsi + dpsio);
//		  stNew.data[iCount] = dMean + ( dpsi + dpsio);
		  //stNew.data[iCount] = dMean - dpsi;
		  iCount++;
		}
	  }
	}

    stNew.iRows = iCount;

	return (stNew);
  }

  /** Method computeTidalPSI()
   *  <p> This method will compute the tidal pressure at a specific time
   *
   *          0.49 * W       1
   *      P = -------- *  --------
   *            a * g     Cw * phi
   *
   *                               _
   *           3   G * M  a   a   |  1
   *       W = - * -----  - * - * |  -- * (3 cos(2*le) - 1) * (3 cos(2*lp) - 1) +
   *           4     D    D   D   |  12
   *						       -
   *                                    sin(lp) * sin(le) * cos(2PI * Fd + phid) +
   *                                                                                 _
   *                                    cos^2(lp) * cos^2(le) * cos[2PI * Fs + phis]  |
   *                                                                                 -
   *
   *       where le   = latitude on earth = wt; where 2PI/w = Earth rotational Period ~24 Hours
   *             lm   = latitude of body with respect to earth
   *             Fd   = Frequecy of the diurnal tide
   *             phid = Phase of the diurnal tide
   *             Fs   = Frequency of the semi-diurnal tide
   *             phis = Phase of the semi-diurnal tide
   *
   *      Tidal Potential Constant:
   *           3   G * M  a   a
   *      Wo = - * -----  - * -  = 2.6282  [m]^2 / [sec]^2
   *           4     D    D   D
   *
   *  @param  iBody    = The type of body to compute pressure
   *  --------------------------------------------------
   *  @param  lstart   = starting time (long value)
   *  @param  ltime    = time (long value)
   *  --------------------------------------------------
   *  @param  longBody = R.A. position of body
   *  @param  distBody = distance between earth and body [m]
   *  --------------------------------------------------
   *  @param  latEarth = latitude of the well
   *  @param  latBody  = latitude of the body with respect to earth
   *  @param  phi      = porosity of the auquifer
   *  @param  Cw       = compressibility of water           1/[GPA]
   *  --------------------------------------------------
   *  @param  dPHIc    = correction
   *  --------------------------------------------------
   *  @return psi      = tide-generating potential
   */

  public static double computeTidalPSI( int iBody, long lstart, long ltime,
                                        double longBody, double distBody,
                                        double latEarth, double latBody, double phi, double Cw,
                                        double dPHIc)
  {
	double psi   = 0.0;                                // [psi]
	double delta = (double) (ltime - lstart) / 1000.0; // [sec]
	double Cwp   = Cw / (GPA * PA_psi);

	double re    = latEarth * Math.PI/180.0;
	double rp    = latBody * Math.PI/180.0;

    double phiM  = Math.PI * (longBody / 12.0);
    double phiW  = Math.PI * (KGS_1_28 / 12.0);
    double phiC  = Math.PI * dPHIc / 180.0;

	double Wo    = 2.6282;                             // [m]^2 / [sec]^2
	double W     = 0.0;                                // [m]^2 / [sec]^2
	double Wl    = 0.0;
	double Wd    = 0.0;
	double Ws    = 0.0;

	double F     = 0.0;
	double d2    = 0.0;

	double dPhase = 0.0;

	double Fe    = HZ_Earth;  // 2 * PI * Earth Rotation Frequency [Hz]

    switch (iBody)
    {
      case _SUN:
        Wo  = 0.75 * (Go * MSun / distBody) * (a/distBody) * (a/distBody);
        break;

      case _MOON:
        Wo  = 0.75 * (Go * M / distBody) * (a/distBody) * (a/distBody);
        break;
	}

    dPhase = phiM + phiW - phiC;
    if (dPhase > 2.0 * Math.PI)
      dPhase = dPhase - 2.0 * Math.PI;
    else if (dPhase < -2.0 * Math.PI)
      dPhase = dPhase + 2.0 * Math.PI;

	Wl  = Wo * (3.0 * Math.cos(2*re)-1) * (3.0 * Math.cos(2*rp)-1) / 12.0;

	F   = 2.0 * Math.PI * Fe;
	Wd  = Wo * Math.sin(re) * Math.sin(rp) * Math.cos(F * delta - dPhase);
//	Wd  = Wo * Math.sin(re) * Math.sin(rp) * Math.cos(F * delta - phiM - phiW + phiC);
//	Wd  = Wo * Math.sin(re) * Math.sin(rp) * Math.cos(F * delta + phiM + phiW - phiC);

	F   = 4.0 * Math.PI * Fe;
	d2  = Math.cos(re) * Math.cos(rp);
	Ws  = Wo * d2 * d2 * Math.cos(F * delta - 2.0 * dPhase);
//	Ws  = Wo * d2 * d2 * Math.cos(F * delta - 2.0 * phiM - 2.0 * phiW + 2.0 * phiC);
//	Ws  = Wo * d2 * d2 * Math.cos(F * delta + 2.0 * phiM + 2.0 * phiW - 2.0 * phiC);

	W   = Wl + Wd + Ws;

	psi = 0.49 * W / ( a * g * phi * Cwp );

	return (psi);
  }

/*
  public static double computeTidalPSI( int iBody, long lstart, long ltime,
                                        double longBody, double distBody,
                                        double latEarth, double latBody, double phi, double Cw,
                                        double dPHIc)
  {
	double psi   = 0.0;                                // [psi]
	double delta = (double) (ltime - lstart) / 1000.0; // [sec]
	double Cwp   = Cw / (GPA * PA_psi);

	double re    = latEarth * Math.PI/180.0;
	double rp    = latBody * Math.PI/180.0;

    double phiM  = Math.PI * (longBody / 12.0);
    double phiW  = 0.0; //Math.PI * (KGS_1_28 / 12.0);
    double phiC  = Math.PI * dPHIc / 180.0;

	double Wo    = 2.6282;                             // [m]^2 / [sec]^2
	double W     = 0.0;                                // [m]^2 / [sec]^2
	double Wl    = 0.0;
	double Wd    = 0.0;
	double Ws    = 0.0;

	double F     = 0.0;
	double d2    = 0.0;
    double dRatio = 0.0;
	double dPhase = 0.0;

	double Fe    = HZ_Earth;  // 2 * PI * Earth Rotation Frequency [Hz]

    switch (iBody)
    {
      case _SUN:
        Wo  = 0.75 * (Go * MSun / distBody) * (a/distBody) * (a/distBody);
        dRatio = (a/distBody);
        break;

      case _MOON:
        Wo  = 0.75 * (Go * M / distBody) * (a/distBody) * (a/distBody);
        dRatio = (a/distBody);
        break;
	}

    dPhase = phiM + phiW - phiC;
    if (dPhase > 2.0 * Math.PI)
      dPhase = dPhase - 2.0 * Math.PI;
    else if (dPhase < -2.0 * Math.PI)
      dPhase = dPhase + 2.0 * Math.PI;

	F   = 2.0 * Math.PI * Fe;
    d2 = Math.sin(re) * Math.sin(rp) + Math.cos(re) * Math.cos(rp) * Math.cos(F * delta - dPhase);
//    Wl = 2.0 * dRatio * (5.0 * d2 * d2 / 3.0 - 1.0) * d2;
    W = 2.0 * d2 * d2 - 0.333333 + Wl;

	psi = 0.49 * W / ( a * g * phi * Cwp );

	return (psi);
  }
*/
  /** Method computeRATimeToDecimal()
   *  <p> This method will convert the RA Time to RA decimal
   *  @param  str = the string time to be converted
   *  @return dRA = the decimal RA value
   */

  public static double computeRATimeToDecimal(String str)
  {
	double dRA        = 0.0;
    String tokens[]   = null;
    String sDelimiter = new String("[:]+");
    String time[]     = { "", "", "" };
    double data       = 0.0;

	if (str.length() > 0)
	{
      tokens = str.split( sDelimiter );
      if (tokens.length == 3)
      {
  	    for (int i=0; i<3; i++)
	    {
	      time[i] = new String(tokens[i]);
	      switch (i)
	      {
			case 0:
			  data = cmn.cmnString.stringToDouble(time[i]);
			  dRA  = data;
			  break;

			case 1:
			  data = cmn.cmnString.stringToDouble(time[i]);
			  dRA  = dRA + data/60.0;
			  break;

			case 2:
			  data = cmn.cmnString.stringToDouble(time[i]);
			  dRA  = dRA + data/3600.0;
			  break;
		  }
	    }
  	  }
	}

	return (dRA);
  }

  /** Method getTimeAsAngle()
   *  <p> This method will get the time String from the date time string
   *  @param  str = the string time to be converted
   *  @return PHI = the angle in degrees
   */

  public static double getTimeAsAngle(String str)
  {
    double PHI        = 0.0; // angle with respect to 00:00 UTC time
    String tokens[]   = null;
    String sDelimiter = new String("[ ]+");
    String stime      = "";
    double dTemp      = 0.0;

	if (str.length() > 0)
	{
      tokens = str.split( sDelimiter );
      if (tokens.length == 2)
      {
		stime = new String(tokens[1]);
	  }

	  if (stime.length() > 0)
	  {
		dTemp = computeRATimeToDecimal(stime);
System.out.println("TIME=" + dTemp);
		PHI = 180.0 * computeRATimeToDecimal(stime) / 12.0;
	  }
    }

    return (PHI);
  }

  /** Method getUT0()
   *  <p> This method will convert the RA Time to RA decimal
   *  @param  str = the string time to be converted
   *  @return dRA = the decimal RA value
   */

  public static long getUT0(String str)
  {
	long   ltime      = (long) 0;
    String tokens[]   = null;
    String sDelimiter = new String("[ ]+");
    String date       = "";
    double data       = 0.0;
    String strn       = "";
    SimpleDateFormat dfm      = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.s");

	if (str.length() > 0)
	{
      tokens = str.split( sDelimiter );
      if (tokens.length == 2)
      {
		date = new String(tokens[0]);
		strn = new String(date + " 00:00:00.0");

        try
        {
          ltime = dfm.parse( strn ).getTime();
     	}
        catch (ParseException e)
        {
          e.printStackTrace();
        }
	  }
	}

	return (ltime);
  }

  /** Method computeRATimeToDecimal()
   *  <p> This method will convert the RA Time to RA decimal
   *  @param  str = the string time to be converted
   *  @return dRA = the decimal RA value
   */

  public static double computeTimeToDecimal(String str, long ltime)
  {
	double dRA        = 0.0;
    String tokens[]   = null;
    String sDelimiter = new String("[ ]+");
    String date[]     = { "", "" };
    double data       = 0.0;
    String strn       = "";
    long   lnew       = (long) 0;
    SimpleDateFormat dfm      = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

	if (str.length() > 0)
	{
      tokens = str.split( sDelimiter );
      if (tokens.length == 2)
      {
		date[0] = new String(tokens[0]);
		strn    = new String(date[0] + " 00:00:00");

        try
        {
          lnew = dfm.parse( strn ).getTime();
     	}
        catch (ParseException e)
        {
          e.printStackTrace();
        }

        dRA = (double) (ltime-lnew);
  	  }
	}

	return (dRA);
  }

  /** Method computeRA()
   *  <p> This method will interpolate the RA decimal value in secs from
   *      the RA array and the pressure time corrected to UTC time.
   *  @param  lstart = starting time (long value)
   *  @param  lend   = ending time (long value)
   *  @param  ltime  = time (long value)
   *  @param  dRA1   = Starting RA value
   *  @param  dRA2   = Ending RA value
   *  @return dRA    = the decimal RA value [secs]
   */

  public static double computeRA(long lstart, long lend, long ltime,
                                 double dRA1, double dRA2)
  {
	double dRA    = 0.0;
	double dRange = (double) (lend  - lstart);
	double diff   = (double) (ltime - lstart);
	double dtemp  = dRA2;

    if (dRA2 < dRA1)
      dtemp = dRA2+24.0;

	dRA = dRA1 + (dtemp - dRA1) * diff / dRange;

    if (dRA >= 24.0)
      dRA = dRA - 24.0;
//System.out.println((new Timestamp(ltime)).toString()+" RA="+dRA);
	return (dRA);
  }

  /** Method computeLatitude()
   *  <p> This method will interpolate the Latitude value
   *  @param  lstart = starting time (long value)
   *  @param  lend   = ending time (long value)
   *  @param  ltime  = time (long value)
   *  @param  dLat1  = Starting Latitude value
   *  @param  dLat2  = Ending Latitude value
   *  @return dLat   = the Latitude value
   */

  public static double computeLatitude(long lstart, long lend, long ltime,
                                       double dLat1, double dLat2)
  {
	double dLat   = 0.0;
	double dRange = (double) (lend  - lstart);
	double diff   = (double) (ltime - lstart);

	dLat = dLat1 + (dLat2 - dLat1) * diff / dRange;

	return (dLat);
  }

  /** Method computeTidalPSI()
   *  <p> This method will compute the tidal pressure at a specific time
   *
   *          0.49 * W       1
   *      P = -------- *  --------
   *            a * g     Cw * phi
   *
   *                               _
   *           3   G * M  a   a   |  1
   *       W = - * -----  - * - * |  -- * (3 cos(2*le) - 1) * (3 cos(2*lp) - 1) +
   *           4     D    D   D   |  12
   *						       -
   *                                    sin(lp) * sin(le) * cos(2PI * Fd + phid) +
   *                                                                                 _
   *                                    cos^2(lp) * cos^2(le) * cos[2PI * Fs + phis]  |
   *                                                                                 -
   *
   *       where le   = latitude on earth = wt; where 2PI/w = Earth rotational Period ~24 Hours
   *             lm   = latitude of moon with respect to earth
   *             Fd   = Frequecy of the diurnal tide
   *             phid = Phase of the diurnal tide
   *             Fs   = Frequency of the semi-diurnal tide
   *             phis = Phase of the semi-diurnal tide
   *
   *      Tidal Potential Constant:
   *           3   G * M  a   a
   *      Wo = - * -----  - * -  = 2.6282  [m]^2 / [sec]^2
   *           4     D    D   D
   *
   *  @param  le       = latitude of the well
   *  @param  lp       = latitude of the moon with respect to earth
   *  @param  phi      = porosity of the auquifer
   *  @param  Cw       = compressibility of water           1/[GPA]
   *  @return W        = array tidal-generating potential
   */

  public static double[] computePSI( double le, double lp, double phi, double Cw )
  {
	double W[]   = {0.0, 0.0, 0.0};                    // [m]^2 / [sec]^2
	double Cwp   = Cw / (GPA * PA_psi);

	double re    = le * Math.PI/180.0;
	double rp    = lp * Math.PI/180.0;

	double Wo    = 2.6282;                             // [m]^2 / [sec]^2
	double d2    = 0.0;
	double psi   = 0.0;

	W[0] = Wo * (3.0 * Math.cos(2*re)-1) * (3.0 * Math.cos(2*rp)-1) / 12.0;
	W[1] = Wo * Math.sin(re) * Math.sin(rp);
	d2   = Math.cos(re) * Math.cos(rp);
	W[2] = Wo * d2 * d2;

	psi = 0.49 / ( a * g * phi * Cwp );

    for (int i=0; i<3; i++)
    {
	  W[i] = (double) ((int) (W[i] * psi * 10000.0) / 10000.0);
	}

	return (W);
  }

  /** Method getCw()
   *  <p> This method will get Compressibility of Water.
   *  @param  degC = degrees Celsius
   *  @return Cw   = Compressibility of Water 1/[GPa]
   */

  public static double getCw( double degC )
  {
	double Cw  = 0.0;
	double Cw2 = 0.0;
	double Cw1 = 0.0;
	double degC2 = 0.0;
	double degC1 = 0.0;
	int    done  = 0;

	for (int i=0; i<CWrows; i++)
	{
	  if ((degC < CW[i][0]) && (done == 0))
	  {
		degC1 = CW[i-1][0];
		degC2 = CW[i][0];
		Cw1   = CW[i-1][1];
		Cw2   = CW[i][1];

		Cw    = Cw2 - (Cw2 - Cw1) * (degC2 - degC) / (degC2 - degC1);
		done  = 1;
	  }
	}

    Cw = (double) ((int) (Cw * 10000.0) / 10000.0);

	return (Cw);
  }

  /** Method degFtodegC()
   *  <p> This method will convert farenheit to celsius
   *  @param  degF = degrees Farenheit
   *  @return degC = degrees Celsius
   */

  public static double degFtodegC(double degF)
  {
	double degC = 0.0;

	degC = 5.0 * (degF - 32.0) / 9.0;
    degC = (double) ((int) (degC * 100.0) / 100.0);

	return (degC);
  }

  /** Method degCtodegF()
   *  <p> This method will convert celsius to farenheit
   *  @param  degC = degrees Celsius
   *  @return degF = degrees Farenheit
   */

  public static double degCtodegF(double degC)
  {
	double degF = 0.0;

	degF = 32.0 + 9.0 * degC / 5.0;
    degF = (double) ((int) (degF * 100.0) / 100.0);

	return (degF);
  }
}

/*
 *  @version 1.1 05/12/2016
 *  @author  John Victorine
 */
