/*
 * @psiPlot.java Version 1.1 06/02/2017
 *
 * Copyright (c) 2017 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package psi.plot;

import java.awt.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.awt.Graphics2D.*;
import java.awt.event.*;
import java.sql.Timestamp;
import java.sql.Date;
import java.text.SimpleDateFormat;
import java.text.ParseException;

import psi.psiPlotStruct;

/** Class psiPlot
 *  <p> This class will create a generic X-Y Plot for an array of data
 *
 *  @version 1.1 06/02/2017
 *  @author  John R. Victorine
 */

public class psiPlot extends Canvas
{
  public static final int _START = 0;
  public static final int _END   = 1;

  public static final int  OFF   = cmn.cmnStruct.OFF;
  public static final int  ON    = cmn.cmnStruct.ON;

  private int              iRawCurve    = ON;  // Raw pressure Data Curve
  private int              iCumulative  = OFF;  // Cumulative pressure Data Curve
  private int              iMeanCurve   = OFF;  // Average pressure Data Curve
  private int              iMedianCurve = OFF;  // Median pressure Data Curve

  public final static int _BOTH    = 0;
  public final static int _RAW     = 1;
  public final static int _AVERAGE = 2;

  private int              iData   = _BOTH;

  private psiPlotStruct  stData  = null;
  private int              iYear   = 0;
  private int              iMonth  = 0;
  private int              iYStart = 0;
  private int              iYEnd   = 0;
  private int              iMStart = 0;
  private int              iMEnd   = 0;
  private int              iMonths = 0;

  private Timestamp        tsStart = null; // Starting Timestamp
  private Timestamp        tsEnd   = null; // Ending Timestamp

  // Plot Dimensions

  private int              iXBegin = 80;  // Left Edge of Plot Area
  private int              iYBegin = 60;  // Top Edge of Plot Area
  private int              iWidth  = 0; //700; // Width of Plot Area
  private int              iHeight = 0; //500; // Height of Plot Area

  // Plot Data

  private int              iRows = 0;  // Total Number of Rows
  private int              iX[];       // Array of X Data converted to Pixels
  private int              iY[];       // Array of Y Data converted to Pixels
  private int              iYa[];      // Array of Y Data converted to Pixels
  private int              iYm[];      // Array of Y Data converted to Pixels
  private int              iYc[];      // Array of Y Data converted to Pixels

  // Plot Variables

  private String           sTitle1     = "";
  private String           sTitle2     = "";
  private String           sTitle3     = "";

  private String           sXAxis      = "";
  private int              iXLog       = 0;
  private double           dXMinimum   = 0.0;
  private double           dXMaximum   = 0.0;
  private double           dXIncrement = 0.0;
  private int              iXCycles    = 1;

  private int              iYLog       = 0;
  private String           sYAxis      = "";
  private double           dYMinimum   = 0.0;
  private double           dYMaximum   = 0.0;
  private double           dYIncrement = 0.0;
  private int              iYCycles    = 1;

  // Filter Extra Data Indicator

  private boolean bFlag = false;

  // Plot Colors

  public static final Color cRED     = new Color( 180,   0,   0 );
  public static final Color cGREEN   = new Color(   0, 180,   0 );
  public static final Color cBLUE    = new Color(   0,   0, 180 );
  public static final Color cCYAN    = new Color(   0, 180, 180 );
  public static final Color cMAGENTA = new Color( 180,   0, 180 );
  public static final Color cYELLOW  = new Color( 180, 180,   0 );
  public static final Color cORANGE  = new Color( 204, 102,   0 );
  public static final Color cBROWN   = new Color( 102, 102,   0 );

  public static final Color COLORS[] =  // Color for Log Curves
      { cRED, cGREEN, cBLUE, cBROWN, cMAGENTA, cORANGE, cYELLOW, cCYAN };

  // Days / Month

  public static final int No_Of_Days[] = { 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

  /** Constructor psiPlot()
   *  <p> This is the Constructor for this class.
   *  @param stData     = The Earthquake Seismic Data Structure
   *  @param sXAxis     = X-Axis Description
   *  @param sYAxis     = Y-Axis Description
   *  @param tsStart    = starting Date & Time
   *  @param tsEnd      = ending Data & Time
   *  @param dMinimum   = Minimum Earthquake Magnitude Plot Value
   *  @param dMaximum   = Maximum Earthquake Magnitude Plot Value
   *  @param dIncrement = Earthquake Magnitude Plat Increment Value
   */

  public psiPlot( psiPlotStruct    stData,
                         String sXAxis,     String sYAxis,
                         Timestamp tsStart, Timestamp tsEnd,
                         double dMinimum,   double dMaximum, double dIncrement )
  {
    this.stData  = stData;

	this.iWidth  = stData.iWidth  - iXBegin; // Set the Actual Width
	this.iHeight = stData.iHeight - iYBegin; // Set the Actual Height

    this.setSize(stData.iWidth, stData.iHeight);
    this.setBackground(Color.white);

    this.iRows   = stData.iRows;

    this.sXAxis  = new String(sXAxis);
    this.sYAxis  = new String(sYAxis);

    this.tsStart = new Timestamp(tsStart.getTime());
    this.tsEnd   = new Timestamp(tsEnd.getTime());

    setDate( this.tsStart );
    iYStart = iYear;
    iMStart = iMonth;

    setDate( this.tsEnd );
    iYEnd   = iYear;
    iMEnd   = iMonth;

    iMonths      = (iYEnd - iYStart) * 12 + (iMEnd - iMStart);
    dXMinimum    = 0;
    dXMaximum    = iMonths;
    dXIncrement  = (double) ((int) (iMonths / 20));

    if (dXIncrement == 0.0) { dXIncrement = 1.0; }

    dYMinimum    = dMinimum;
    dYMaximum    = dMaximum;
    dYIncrement  = dIncrement;

    // Convert Raw Data to Pixels

    convertDataToPixels();
  }

  /* =============================================================== *
   * ----------------------- DELETE METHODS ------------------------ *
   * =============================================================== */

  /** Method close()
   * <p> This method will set strings and arrays to null so Java can
   *     automatically clear memory
   */

  public void close()
  {
    stData   = null;

    sTitle1  = null;
    sTitle2  = null;
    sTitle3  = null;

    sXAxis   = null;
    sYAxis   = null;

    tsStart  = null; // Starting Timestamp
    tsEnd    = null; // Ending Timestamp

    iX       = null; // Array of X Data converted to Pixels
    iY       = null; // Array of Y Data converted to Pixels
    iYa      = null;
    iYm      = null;
    iYc      = null;
  }

  /* ================================================================== *
   * -------------------------- GET METHODS --------------------------- *
   * ================================================================== */

  /** Method getPlotWidth()
   * <p> This method will return the Plot Width
   * @return iWidth = The Plot Width
   */

  public int getPlotWidth()  { return (stData.iWidth); }

  /** Method getPlotHeight()
   * <p> This method will return the Plot Height
   * @return iHeight = The Plot Height
   */

  public int getPlotHeight() { return (stData.iHeight); }

  /** Method getImage()
   * <p> This method will convert this plot to a buffered image
   * @return image = profile plot buffered image
   */

  public BufferedImage getImage()
  {
    BufferedImage image = null;
    int w = this.getWidth();
    int h = this.getHeight();

    image = new BufferedImage( w, h, BufferedImage.TYPE_INT_RGB );

    Graphics2D g2 = image.createGraphics();
    g2.setColor( Color.white );
    g2.fillRect( 0, 0, w, h );
    this.paint(g2);
    g2.dispose();

    return (image);
  }

  /** Method getTimestamp()
   *  <p> This method will convert the Date Time String into a Timestamp.
   *  @param iY = Year
   *  @param iM = iMonth
   */

  private Timestamp getTimestamp( int iY, int iM )
  {
	Timestamp        ts    = null;
    SimpleDateFormat dfm   = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSSSSS");
    String           sText = "";
    long             ltime = (long) 0;


    sText = new String( iY + "-" + iM + "-" + "01 00:00:00.0");

    try
    {
      ltime = dfm.parse(sText).getTime();
      ts    = new Timestamp( ltime );
	}
    catch (ParseException e)
    {
      e.printStackTrace();
    }

	return (ts);
  }

  /** Method getTimestamp()
   *  <p> This method will convert the Date Time String into a Timestamp.
   *  @param iY = Year
   *  @param iM = iMonth
   */

  private Timestamp getTimestamp( int iY, int iM, int iD )
  {
	Timestamp        ts    = null;
    SimpleDateFormat dfm   = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSSSSS");
    String           sText = "";
    long             ltime = (long) 0;


    sText = new String( iY + "-" + iM + "-" + iD + " 00:00:00.0");

    try
    {
      ltime = dfm.parse(sText).getTime();
      ts    = new Timestamp( ltime );
	}
    catch (ParseException e)
    {
      e.printStackTrace();
    }

	return (ts);
  }

  /* ================================================================== *
   * -------------------------- SET METHODS --------------------------- *
   * ================================================================== */

  public void setData( psiPlotStruct stData,
                       String sXAxis,     String sYAxis,
                       Timestamp tsStart, Timestamp tsEnd,
                       double dMinimum,   double dMaximum, double dIncrement )
  {
    this.stData  = stData;

    this.iRows   = stData.iRows;

    this.sXAxis  = new String(sXAxis);
    this.sYAxis  = new String(sYAxis);

    this.tsStart = new Timestamp(tsStart.getTime());
    this.tsEnd   = new Timestamp(tsEnd.getTime());

    setDate( this.tsStart );
    iYStart = iYear;
    iMStart = iMonth;

    setDate( this.tsEnd );
    iYEnd   = iYear;
    iMEnd   = iMonth;

    iMonths      = (iYEnd - iYStart) * 12 + (iMEnd - iMStart);
    dXMinimum    = 0;
    dXMaximum    = iMonths;
    dXIncrement  = (double) ((int) (iMonths / 20));

    if (dXIncrement == 0.0) { dXIncrement = 1.0; }

    dYMinimum    = dMinimum;
    dYMaximum    = dMaximum;
    dYIncrement  = dIncrement;

    // Convert Raw Data to Pixels

    convertDataToPixels();
    repaint();
  }

  /** Method setDate()
   *  <p> This method will set the Date Value into either the
   *      Start Date or End Date.
   *  @param ts    = timestamp
   */

  private void setDate(Timestamp ts)
  {
    int i = 0;
    String tokens[]    = null;
    String sDelimiter  = new String("[ ]+");
    String sDelimiter1 = new String("[-]+");
    String str         = "";
    String sDate       = "";
    String date[]      = { "", "", "" };
    int    iContinue   = 0;
    double d           = 0.0;
    int    id          = 0;

    if (ts != null)
    {
      str = new String(ts.toString());
      tokens = str.split( sDelimiter );
      if (tokens.length > 0)
        sDate = new String(tokens[0].trim());

      tokens = sDate.split( sDelimiter1 );
      if (tokens.length == 3)
      {
  	    for (i=0; i<3; i++)
	    {
	      date[i] = new String(tokens[i]);
	    }

        iYear  = cmn.cmnString.stringToInt(date[0]);
	    iMonth = cmn.cmnString.stringToInt(date[1]);
      }
    }
  }

  /** Method setTitle1()
   * <p> This method will set the first user title
   * @param sText = the first title
   */

  public void setTitle1(String sText)
  {
    this.sTitle1 = new String(sText);
    repaint();
  }

  /** Method setTitle2()
   * <p> This method will set the second user title
   * @param sText = the second title
   */

  public void setTitle2(String sText)
  {
    this.sTitle2 = new String(sText);
    repaint();
  }

  /** Method setTitle3()
   * <p> This method will set the third user title
   * @param sText = the third title
   */

  public void setTitle3(String sText)
  {
    this.sTitle3 = new String(sText);
    repaint();
  }

  /** Method setXAxis()
   * <p> This method will reset the X-Axis Parameters
   * @param sText = The Axis Label
   * @param iGrid = The type of grid to create
   * @param dMax  = The Maximum axis value
   * @param dMin  = The Minimum axis value
   * @param dInc  = The Increment value on the axis - Linear Grid only
   * @param iCycle = The Number of Cycles - Log Grid only
   */

  public void setXAxis(String sText,
                       int    iGrid,
                       double dMax,
                       double dMin,
                       double dInc,
                       int    iCycle)
  {
    this.sXAxis      = new String(sText);
    this.iXLog       = iGrid;
    this.dXMaximum   = dMax;
    this.dXMinimum   = dMin;
    this.dXIncrement = dInc;
    this.iXCycles    = iCycle;

    convertDataToPixels();
    repaint();
  }

  /** Method setYAxis()
   * <p> This method will reset the Y-Axis Parameters
   * @param sText = The Axis Label
   * @param iGrid = The type of grid to create
   * @param dMax  = The Maximum axis value
   * @param dMin  = The Minimum axis value
   * @param dInc  = The Increment value on the axis - Linear Grid only
   * @param iCycle = The Number of Cycles - Log Grid only
   */

  public void setYAxis(String sText,
                       int    iGrid,
                       double dMax,
                       double dMin,
                       double dInc,
                       int    iCycle)
  {
    this.sYAxis      = new String(sText);
    this.iYLog       = iGrid;
    this.dYMaximum   = dMax;
    this.dYMinimum   = dMin;
    this.dYIncrement = dInc;
    this.iYCycles    = iCycle;

    convertDataToPixels();
    repaint();
  }

  /** Method setXAxis()
   *  @param tsStart = starting Date & Time
   *  @param tsEnd   = ending Data & Time
   */

  public void setXAxis( Timestamp tsStart, Timestamp tsEnd )
  {
    this.tsStart = new Timestamp(tsStart.getTime());
    this.tsEnd   = new Timestamp(tsEnd.getTime());

    setDate( this.tsStart );
    iYStart = iYear;
    iMStart = iMonth;

    setDate( this.tsEnd );
    iYEnd   = iYear;
    iMEnd   = iMonth;

    iMonths      = (iYEnd - iYStart) * 12 + (iMEnd - iMStart);
    dXMinimum    = 0;
    dXMaximum    = iMonths;
    dXIncrement  = (double) ((int) (iMonths / 10));

    // Convert Raw Data to Pixels

    convertDataToPixels();
    repaint();
  }

  /** Method setFlag()
   *  <p> Set a Flag to filter extra data or not.
   *  @param bFlag = true to filter data or false not to filter extra data.
   */

  public void setFlag(boolean bFlag)
  {
    this.bFlag = bFlag;
    repaint();
  }

  /** Method setRawDataCurve()
   *  <p> This method will set the Plot Raw Data Curve ON or OFF
   *  @param iValue = the value to set for the Raw Data
   */

  public void setRawDataCurve( int iValue )
  {
	this.iRawCurve = iValue;
	repaint();
  }

  /** Method setCumulativeCurve()
   *  <p> This method will set the Plot Cumulative Data Curve ON or OFF
   *  @param iValue = the value to set for the Raw Data
   */

  public void setCumulativeCurve( int iValue )
  {
    this.iCumulative = iValue;  // Cumulative pressure Data Curve
	repaint();
  }

  /** Method setAverageCurve()
   *  <p> This method will set the Plot Mean Data Curve ON or OFF
   *  @param iValue = the value to set for the Raw Data
   */

  public void setAverageCurve( int iValue )
  {
    this.iMeanCurve = iValue;  // Average pressure Data Curve
	repaint();
  }

  /** Method setMedianCurve()
   *  <p> This method will set the Plot Median Data Curve ON or OFF
   *  @param iValue = the value to set for the Raw Data
   */

  public void setMedianCurve( int iValue )
  {
    this.iMedianCurve = iValue;  // Median pressure Data Curve
	repaint();
  }


  /* ================================================================== *
   * ------------------------ SUPPORT METHODS ------------------------- *
   * ================================================================== */

  /** Method void convertDataToPixels()
   *  <p> This method will set the Plot Data
   *  @param ID = the data set being processed.
   */

  private void convertDataToPixels()
  {
    int    i      = 0;
    int    iRatio = 0;
    double dXMin  = 0;
    double dXMax  = 0;
    double dYMin  = 0;
    double dYMax  = 0;
    double dXi    = 0.0;
    double dYi    = 0.0;

    long   lstart = tsStart.getTime();
    long   lend   = tsEnd.getTime();
    long   ltime  = (long) 0;
    double dRatio = 0.0;

//    iRows = stPlot.iRows;
    iX  = new int [iRows];
    iY  = new int [iRows];
    iYa = new int [iRows];
    iYm = new int [iRows];
    iYc = new int [iRows];

    // Set the X-Axis Data

    for (i=0; i<iRows; i++)
    {
	  ltime  = stData.ts[i].getTime();
	  dRatio = (double) (iWidth * (ltime - lstart) / (lend - lstart));
      iX[i]  = iXBegin + (int) dRatio;
    }

    // Set the Y-Axis Data

    // IF the Y-Axis Data is LINEAR THEN
    if (iYLog == 0)
    {
      for (i=0; i<iRows; i++)
      {
		dYi    = stData.dRaw[i];
        iRatio = (int)((double)iHeight * (dYi-dYMinimum)/(dYMaximum-dYMinimum));
        iY[i]  = iHeight + iYBegin - iRatio;
      }
    }
    // IF the Y-Axis Data is LOG THEN
    else
    {
      dYMin = Math.log(dYMinimum) / Math.log(10.0);
      dYMax = Math.log(dYMaximum) / Math.log(10.0);

      for (i=0; i<iRows; i++)
      {
		dYi    = Math.log(stData.dRaw[i])/Math.log(10.0);
        iRatio = (int)((double)iHeight * (dYi-dYMin) / (dYMax - dYMin));
        iY[i]  = iHeight + iYBegin - iRatio;
      }
    }
  }

  /* ================================================================== *
   * -------------------------- DRAW METHODS -------------------------- *
   * ================================================================== */

  /** Method drawTitles()
   *  <p> This method will draw the titles of the Plot.
   *  @param g - Graphics pointer.
   */

  public void drawTitles(Graphics g)
  {
    int iTemp = 0;

    g.setColor(Color.black);

    g.setFont( new Font("Monospaced", Font.BOLD, 14) );

    if (sTitle1 != null)
    {
      iTemp = (int) (Math.abs(iWidth - (6 * sTitle1.length())));
      g.drawString(sTitle1, iTemp / 2, 10);
    }

    if (sTitle2 != null)
    {
      iTemp = (int) (Math.abs(iWidth - (6 * sTitle2.length())));
      g.drawString(sTitle2, iTemp / 2, 30);
    }

    if (sTitle3 != null)
    {
      iTemp = (int) (Math.abs(iWidth - (6 * sTitle3.length())));
      g.drawString(sTitle3, iTemp / 2, 50);
    }
  }

  /** Method drawLabels()
   *  <p> This method will draw the Linear Plot Labels.
   *  @param g - Graphics pointer.
   */

  public void drawLabels(Graphics g)
  {
    int ivalue;

    g.setColor(Color.black);

    if (sXAxis != null)
    {
      ivalue = (iWidth+100)/2 - 3 * (sXAxis).length();
      g.drawString(sXAxis, ivalue, iYBegin+iHeight+35);
    }

    if (sYAxis != null)
    {
      ivalue = (iHeight + 100) / 2 - 3 * (sYAxis).length();

      Graphics2D g2 = (Graphics2D) g;
      g2.rotate( (Math.PI / 2.0));
      g2.drawString(sYAxis, ivalue, -10);
      g2.rotate( -1.0 * (Math.PI / 2.0));
    }
  }

  /** Method drawGrid()
   *  <p> This method will draw the Grid Lines
   *  @param g - Graphics pointer.
   */

  public void drawGrid(Graphics g)
  {
    if (iXLog == 0) //seismic.quakePlotXYStruct.LINEAR)
      drawLinearXAxisGrid(g);
    else
      drawLogXAxisGrid(g);

    if (iYLog == 0) //seismic.quakePlotXYStruct.LINEAR)
      drawLinearYAxisGrid(g);
    else
      drawLogYAxisGrid(g);
  }

  /** Method drawLinearXAxisGrid()
   *  <p> This method will draw the Linear X-Axis Grid Lines
   *  @param g - Graphics pointer.
   */

  public void drawLinearXAxisGrid(Graphics g)
  {
    int    i      = 0;
    int    j      = 0;
    int    idel   = 0;
    int    iRatio = 0;
    int    iXinc  = 0;
    double dx     = 0.0;
    double del    = 0.0;

    int    iDays  = 0;
    int    idiff  = 0;

    int    idraw  = 0;
    long   lstart = tsStart.getTime();
    long   lend   = tsEnd.getTime();
    long   ltime  = (long) 0;
    double dRatio = 0.0;
    int    iM     = 0;
    int    iY     = 0;
    Timestamp ts  = null;

    iY   = iYStart;
    idel = iMonths/12;
    iM   = iMStart;

    switch (iYEnd-iYStart)
    {
  	  case 0:
	   iDays = No_Of_Days[iM];
	   idiff = 1;
	   break;
	  case 1:
	   iDays = 4;
	   idiff = 7;
	   break;
	  default:
	   iDays = 0;
	   break;
    }

    if (idel == 0) { idel = 1; }

    for (i=0; i<iMonths+1; i++)
    {
      for (int k=0; k<iDays; k++)
	  {
        ts = getTimestamp( iY, iM, idiff*(k+1) );
        ltime  = ts.getTime();

	    dRatio = (double) (iWidth * (ltime - lstart) / (lend - lstart));
        idraw  = iXBegin + (int) dRatio;

        if (idraw <= iXBegin + iWidth)
        {
          g.setColor(new Color(230, 230, 255));
          g.drawLine(idraw, iYBegin, idraw, iYBegin+iHeight);
	    }
	  }

	  if ((i==0) || (i==iMonths))
		g.setColor(Color.black);
	  else
        g.setColor(Color.lightGray);

	  if (iM > 12)
	  {
		iM = 1;
		iY++;
		g.setColor(Color.black);
	  }

      ts = getTimestamp( iY, iM );
      ltime  = ts.getTime();

	  dRatio = (double) (iWidth * (ltime - lstart) / (lend - lstart));
      idraw  = iXBegin + (int) dRatio;

      if ((i==0) || (iM == 1))
      {
        g.drawString(""+iY, idraw, iYBegin + iHeight + 28);
	  }

      if (idraw <= iXBegin + iWidth)
      {
        g.drawLine(idraw, iYBegin, idraw, iYBegin+iHeight);

        g.setColor(Color.black);
        g.drawString(""+iM, idraw, iYBegin + iHeight + 15);
	  }

	  iM = iM + idel;
	}
  }

  /** Method drawLogXAxisGrid()
   *  <p> This method will draw the Log X-Axis Grid Lines
   *  @param g - Graphics pointer.
   */

  public void drawLogXAxisGrid(Graphics g)
  {
    int    i, j;
    int    l=1;
    int    k=0;
    int    iXinc;
    double dLabel = 0.0;
    double del    = 0.0;
    int    jXinc;
    double dTemp;
    double dXinc;

    iXinc = 1 + iWidth / iXCycles;

    g.setColor(Color.darkGray);
    g.drawLine(iXBegin, iYBegin, iXBegin, iYBegin+iHeight);

    for (i=iXBegin+iXinc; i<iXBegin+iWidth+10; i+=iXinc)
    {
      g.setColor(Color.darkGray);
      g.drawLine(i, iYBegin, i, iYBegin+iHeight);

      g.setColor(Color.lightGray);
      for (j=2; j<10; j++)
      {
        dXinc = (double) j;
        dTemp = 1.0+(double) iXinc * Math.log(dXinc) / Math.log(10.0);
        jXinc = i - iXinc + (int) dTemp;

        g.drawLine(jXinc, iYBegin, jXinc, iYBegin+iHeight);
      }
    }

    g.setFont( new Font("Monospaced", Font.BOLD, 10) );
    g.setColor(Color.black);
    k=1;
    del = dXIncrement;
    g.drawString("" + dXMinimum, iXBegin-10, iYBegin+iHeight+15);
    for (i=iXBegin+iXinc; i<iXBegin+iWidth+10; i+=iXinc)
    {
      g.setColor(Color.darkGray);

      dLabel = (double) k * 10.0 * dXMinimum;
      k = k * 10;

      g.setColor(Color.black);
      g.drawString("" + dLabel, i-10, iYBegin+iHeight+15);
      del = dXIncrement + del;
    }
  }

  /** Method drawLinearYAxisGrid()
   *  <p> This method will draw the Linear Y-Axis Grid Lines
   *  @param g - Graphics pointer.
   */

  public void drawLinearYAxisGrid(Graphics g)
  {
    int    i = 0;
    int    j = 0;
    int    idraw  = 0;
    int    idel   = 0;
    int    iRatio = 0;
    int    iYinc  = 0;
    double dy     = 0.0;
    double del    = 0.0;

    iYinc = (int) ((double)iHeight * dYIncrement / (dYMaximum - dYMinimum));

    idraw = iYBegin + iHeight;
    while (idraw > iYBegin-5)
    {
      dy = dYMinimum + (double) i * dYIncrement;
      iRatio = (int)((double)iHeight * (dy - dYMinimum) / (dYMaximum - dYMinimum));
      idraw = iHeight + iYBegin - iRatio;

      if (idraw > iYBegin-5)
      {
        g.setColor(Color.darkGray);
        g.drawLine(iXBegin, idraw, iXBegin+iWidth, idraw);

        g.setColor(Color.black);
        g.setFont( new Font("Monospaced", Font.BOLD, 10) );
        dy = Math.round(dy*1000)/1000.0;
        g.drawString("" + dy, iXBegin-45, idraw);

        for (j=1; j<5; j++)
        {
          idel = idraw - (j * iYinc) / 5;
          if (idel > iYBegin)
          {
            g.setColor(Color.lightGray);
            g.drawLine(iXBegin, idel, iXBegin+iWidth, idel);
          }
        }
      }

      i++;
    }
  }

  /** Method drawLogYAxisGrid()
   *  <p> This method will draw the Log X-Axis Grid Lines
   *  @param g - Graphics pointer.
   */

  public void drawLogYAxisGrid(Graphics g)
  {
    int    i = 0;
    int    j = 0;
    int    k = 1;
    int    iYinc  = 0;
    int    jXinc  = 0;
    double dy     = 0.0;
    double dTemp  = 0.0;
    double dXinc  = 0.0;

    iYinc = 1 + iHeight / iYCycles;

    for (i=iYBegin+iHeight; i>iYBegin; i-=iYinc)
    {
      k = i;
      g.setColor(Color.darkGray);
      g.drawLine(iXBegin, i, iXBegin+iWidth, i);

      g.setColor(Color.lightGray);
      for (j=2; j<10; j++)
      {
        dXinc = (double) j;
        dTemp = 1.0+(double) iYinc * Math.log(dXinc) / Math.log(10.0);
        jXinc = i - (int) dTemp;

        g.drawLine(iXBegin, jXinc, iXBegin+iWidth, jXinc);
      }
    }

    g.setColor(Color.darkGray);
    g.drawLine(iXBegin, k-iYinc, iXBegin+iWidth, k-iYinc);

    g.setFont( new Font("Monospaced", Font.BOLD, 10) );
    g.setColor(Color.black);
    k=1;
    g.drawString("" + dYMinimum, iXBegin-45, iYBegin+iHeight);
    for (i=iYBegin+iHeight; i>iYBegin; i-=iYinc)
    {
      dy = (double) k * 10.0 * dYMinimum;
      k = k * 10;
      g.drawString("" + dy, iXBegin-45, i-iYinc);
    }
  }

  /** Method drawPlotData()
   * <p> This method will plot the data
   *  @param g - Graphics pointer.
   */

  public void drawPlotData(Graphics g)
  {
	int iYo       = 0;
    int iX1       = 0;
    int iX2       = 0;
    int iY1       = 0;
    int iY2       = 0;
    int iContinue = 0;

    g.setColor( COLORS[1] );

    if (iRawCurve == ON)
    {
      for (int i=0; i<iRows; i++)
      {
        iX1 = iX[i];
        iY1 = iY[i];

        if ((iX1 >= iXBegin) && (iX1 <= iXBegin + iWidth)  &&
            (iY1 >= iYBegin) && (iY1 <= iYBegin + iHeight))
        {
          g.fillRect(iX1-2, iY1-2, 4, 4);
		}
  	  }

      g.setFont( new Font("Monospaced", Font.BOLD, 12) );

      iYo = (iMeanCurve+iMedianCurve+iCumulative) * 15;

      g.fillRect(iWidth-50+4, iYBegin-9-iYo, 4, 4);
	  g.drawString("Raw Data", iWidth-30, iYBegin-5-iYo);
    }
  }

  /** Method drawAverageData()
   * <p> This method will plot the data
   *  @param g - Graphics pointer.
   */

  public void drawAverageData(Graphics g)
  {
	int iY        = 0;
    int iX1       = 0;
    int iX2       = 0;
    int iY1       = 0;
    int iY2       = 0;
    int iContinue = 0;

    g.setColor( COLORS[0] );

    if (iMeanCurve == ON)
    {
      for (int i=0; i<iRows-1; i++)
      {
        iX1 = iX[i];
        iY1 = iYa[i];
        iX2 = iX[i+1];
        iY2 = iYa[i+1];

        if ((iX1 >= iXBegin) && (iX1 <= iXBegin + iWidth)  &&
            (iY1 >= iYBegin) && (iY1 <= iYBegin + iHeight) &&
            (iX2 >= iXBegin) && (iX2 <= iXBegin + iWidth)  &&
            (iY2 >= iYBegin) && (iY2 <= iYBegin + iHeight))
        {
		  g.drawLine(iX1, iY1, iX2, iY2);
//        g.fillRect(iX1, iY1, 4, 4);
  	    }
	  }

      g.setFont( new Font("Monospaced", Font.BOLD, 12) );

      iY = (iMedianCurve+iCumulative) * 15;

	  g.drawLine(iWidth-50, iYBegin-5-iY, iWidth-35, iYBegin-5-iY);
	  g.drawString("Average", iWidth-30, iYBegin-5-iY);
    }
  }

  /** Method drawMedianData()
   * <p> This method will plot the data
   *  @param g - Graphics pointer.
   */

  public void drawMedianData(Graphics g)
  {
	int iY        = 0;
    int iX1       = 0;
    int iX2       = 0;
    int iY1       = 0;
    int iY2       = 0;
    int iContinue = 0;

    g.setColor( COLORS[6] );

    if (iMedianCurve == ON)
    {
      for (int i=0; i<iRows-1; i++)
      {
        iX1 = iX[i];
        iY1 = iYm[i];
        iX2 = iX[i+1];
        iY2 = iYm[i+1];

        if ((iX1 >= iXBegin) && (iX1 <= iXBegin + iWidth)  &&
            (iY1 >= iYBegin) && (iY1 <= iYBegin + iHeight) &&
            (iX2 >= iXBegin) && (iX2 <= iXBegin + iWidth)  &&
            (iY2 >= iYBegin) && (iY2 <= iYBegin + iHeight))
        {
  		  g.drawLine(iX1, iY1, iX2, iY2);
//        g.fillRect(iX1, iY1, 4, 4);
    	}
      }

      g.setFont( new Font("Monospaced", Font.BOLD, 12) );

      iY = (iCumulative) * 15;

	  g.drawLine(iWidth-50, iYBegin-5-iY, iWidth-35, iYBegin-5-iY);
	  g.drawString("Median", iWidth-30, iYBegin-5-iY);
    }
  }

  /** Method drawCumulativeData()
   * <p> This method will plot the data
   *  @param g - Graphics pointer.
   */

  public void drawCumulativeData(Graphics g)
  {
	int iY        = 0;
    int iX1       = 0;
    int iX2       = 0;
    int iY1       = 0;
    int iY2       = 0;
    int iContinue = 0;

    g.setColor( COLORS[3] );

    if (iCumulative == ON)
    {
      for (int i=0; i<iRows-1; i++)
      {
        iX1 = iX[i];
        iY1 = iYc[i];
        iX2 = iX[i+1];
        iY2 = iYc[i+1];

        if ((iX1 >= iXBegin) && (iX1 <= iXBegin + iWidth)  &&
            (iY1 >= iYBegin) && (iY1 <= iYBegin + iHeight) &&
            (iX2 >= iXBegin) && (iX2 <= iXBegin + iWidth)  &&
            (iY2 >= iYBegin) && (iY2 <= iYBegin + iHeight))
        {
	      g.drawLine(iX1, iY1, iX2, iY2);
//          g.fillRect(iX1, iY1, 4, 4);
	    }
  	  }

      g.setFont( new Font("Monospaced", Font.BOLD, 12) );

	  g.drawLine(iWidth-50, iYBegin-5-iY, iWidth-35, iYBegin-5-iY);
	  g.drawString("Cumulative", iWidth-30, iYBegin-5-iY);
    }
  }

  /** Method draw()
   *  <p> This method will create the Linear Plot.
   *  @param g - Graphics pointer.
   */

  public void draw(Graphics g)
  {
    drawTitles(g);
    drawLabels(g);
    drawGrid(g);
    g.setColor(Color.black);

    drawPlotData(g);
    drawAverageData(g);
    drawMedianData(g);
    drawCumulativeData(g);
  }

  /** Method paint()
   *  <p> This method will create the Linear Plot.
   *  @param g - Graphics pointer.
   */

  public void paint(Graphics g)
  {
	draw(g);
  }
}

/*
 *  @version 1.1 06/02/2017
 *  @author  John Victorine
 */
