/*
 * @seisPlotFrame.java Version 1.1 03/05/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package seismic.gui;

import java.util.Observable;
import java.sql.Timestamp;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import cmn.cmnStruct;                // Global Common Data Structure

import iqstrat.iqstratHeadersStruct; // Headers Data Structure

import seismic.seisStruct;           // Seismic Data Structure
import seismic.gui.seisPlotPanel;    // Seismic Data Plot Panel

//import psi.psiTidalStruct;           // Tidal Parameters Data Structure

/** Class seisPlotFrame
 *  <p> This Class will create the Lithology Plot Frame.
 *
 *  @version 1.1 03/05/2015
 *  @author  John R. Victorine
 */

public class seisPlotFrame extends JFrame implements ActionListener
{
  // Plot Location & Size

  private int iFrameX      = -1;
  private int iFrameY      = -1;
  private int iFrameWidth  = -1;
  private int iFrameHeight = -1;

  // Screen Width

  private int iControlWidth = 250;
  private int iScreenWidth  = 0;
  private int iScreenHigh   = 0;

  private iqstratHeadersStruct stHeader = null;
  private seisPlotPanel        plot     = null;

  // Name Textfield

  private JTextField txtName = new JTextField();

  /** Constructor seisPlotFrame()
   *  <p> This is the Constructor for this class.
   *  @param stCMN        = Common Data Structure
   *  @param tsStart      = The starting date
   *  @param tsEnd        = The ending date
   *  @param iScale       = The time scale
   *  @param iSelected    = The selected tracks for plotting
   *  @param iWidth       = Control Dialog Width
   *  @param iFrameX      = The Last X-coordinate when dialog closed
   *  @param iFrameY      = The Last Y-coordinate when dialog closed
   *  @param iFrameWidth  = The Frame Width when dialog closed
   *  @param iFrameHeight = The Frame Height when dialog closed
   */

  public seisPlotFrame( cmnStruct stCMN,
                        Timestamp tsStart,
                        Timestamp tsEnd,
                        int       iScale,
                        int       iSelected[],
                        int       iWidth,
                        int       iFrameX,
                        int       iFrameY,
                        int       iFrameWidth,
                        int       iFrameHeight )
  {
    try
    {
      this.iControlWidth  = iWidth;
      this.iFrameX        = iFrameX;
      this.iFrameY        = iFrameY;
      this.iFrameWidth    = iFrameWidth;
      this.iFrameHeight   = iFrameHeight;

      Cursor normalCursor = new Cursor(Cursor.DEFAULT_CURSOR);
      Cursor mouseCursor  = new Cursor(Cursor.HAND_CURSOR);

      jbInit(stCMN, tsStart, tsEnd, iScale, iSelected, normalCursor, mouseCursor);

      addWindowListener( new seisPlotFrame_WindowListener() );
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @param stCMN        = Common Data Structure
   *  @param tsStart      = The starting date
   *  @param tsEnd        = The ending date
   *  @param iScale       = The time scale
   *  @param iSelected    = The selected tracks for plotting
   *  @param normalCursor = Normal Cursor
   *  @param mouseCursor  = Hand Cursor
   *  @throws Exception
   */

  private void jbInit( cmnStruct stCMN,
                       Timestamp tsStart,
                       Timestamp tsEnd,
                       int       iScale,
                       int       iSelected[],
                       Cursor    normalCursor,
                       Cursor    mouseCursor ) throws Exception
  {
    String    sTitle  = "Plot";
    Dimension d       = null;

    this.setTitle(sTitle);
    this.getContentPane().setLayout(new BorderLayout());

    plot = new seisPlotPanel( stCMN, tsStart, tsEnd, iScale, iSelected,
                              normalCursor, mouseCursor);

    this.getContentPane().add(plot, BorderLayout.CENTER);

    // Display the Frame

    d  = Toolkit.getDefaultToolkit().getScreenSize();
    iScreenWidth = d.width;
    iScreenHigh  = d.height-50;

    if ((iFrameX == -1) && (iFrameY == -1))
    {
      this.setSize(500, iScreenHigh);
//      this.setSize(iScreenWidth-iControlWidth, iScreenHigh);
//      this.setSize(930, iScreenHigh);
//      this.setLocation(iControlWidth+1, 1);
      this.setLocation(iControlWidth+1, 1);
    }
    else
    {
      this.setSize(iFrameWidth, iFrameHeight);
      this.setLocation(iFrameX, iFrameY);
    }

    this.setVisible(true);
  }

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    stHeader = null;

    txtName  = null;

    if (plot != null)
      plot.close();
    plot = null;

    dispose();
  }

  /** METHOD seisPlotFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class seisPlotFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
      close();
//      if (notifier != null)
//        notifier.notifyObservers(new String("Close PROFILE Plot"));
    }
  }

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)  { }

  /** Method refresh()
   *  <p> This method will refresh the plot
   */

  public void refresh() { if (plot != null) { plot.refresh(); } }

  /* ======================================================================= *
   * ----------------------------- GET METHODS ----------------------------- *
   * ======================================================================= */

  /** Method getPlotWidth()
   * <p> This method will return the Plot Width
   * @return iWidth = The Plot Width
   */

  public int  getPlotWidth()  { return ( plot.getPlotWidth() ); }

  /** Method getPlotHeight()
   * <p> This method will return the Plot Height
   * @return iHeight = The Plot Height
   */

  public int getPlotHeight() { return ( plot.getPlotHeight() ); }

  /** Method getImage()
   * <p> This method will convert this plot to a buffered image
   * @return image = profile plot buffered image
   */

  public BufferedImage getImage()
  {
    BufferedImage image = null;

    if (plot != null) image = plot.getImage();

    return (image);
  }

  /* ======================================================================= *
   * ----------------------------- SET METHODS ----------------------------- *
   * ======================================================================= */

  /** Method setObservable( Observable notifier )
   * <p> This method will set the Observerable Object
   * @param notifier = the observable object
   */

  public void setObservable( Observable notifier )
  {
    if (plot != null) plot.setObservable( notifier );
  }

  /** Method setTextfield( JTextField txt )
   * <p> This method will set the depth textfield
   * @param txtDepth = The Depth Text Field
   */

  public void setTextfield(JTextField txt)
  {
    if (plot != null) plot.setTextfield(txt);
  }

  /** Method setTextfieldName( JTextField txtName )
   * <p> This method will set the name textfield
   * @param txtName = The name Text Field
   */

  public void setTextfieldName(JTextField txtName) { this.txtName = txtName; }

  /** Method setScale(int iScale, double dMaximum, double dMinimum)
   * <p> This method will set the Plot scale in the
   *     Cross Plot Data List Structure
   * @param iScale    = The Time Track Scale identifier
   * @param tsMaximum = The Maximum Timestamp
   * @param tsMinimum = The Minimum Timestamp
   */

  public void setScale(int iScale, Timestamp tsMaximum, Timestamp tsMinimum)
  {
    if (plot != null)
    {
      plot.setScale( iScale, tsMaximum, tsMinimum );
      plot.updateUI();
    }
  }

  public int getLocationX()   { return ( this.getLocation().x ); }
  public int getLocationY()   { return ( this.getLocation().y ); }
  public int getFrameWidth()  { return ( this.getSize().width ); }
  public int getFrameHeight() { return ( this.getSize().height ); }

  /* ----------------------------------------------------------------------- *
   * ----------------- Available Tracks Selected for view ------------------ *
   * ----------------------------------------------------------------------- */

  /** Method setSelected( int iSelected[] )
   * <p> This method will change the selected tracks
   *  @param iSelected  = The Tracks that will be displayed
   */

  public void setSelected( int iSelected[] )
  {
    if (plot != null) plot.setSelected( iSelected );
  }

  /* ----------------------------------------------------------------------- *
   * ----------------------------- Seismic Data ---------------------------- *
   * ----------------------------------------------------------------------- */

  /** Method setHeaders( iqstratHeadersStruct st )
   * <p> This method will set the Headers data structure
   * @param st = the headers data structure
   */

  public void setHeaders( iqstratHeadersStruct st )
  {
    String sTitle1 = "";
    String sTitle2 = "";

    this.stHeader = st;

    if (this.stHeader != null)
    {
//      this.stHeader.iType = stStruct.iDataType;
//      if (stStruct.iDataType == iqstrat.iqstratStruct._WELL_DATA)
//      {
        if (this.stHeader.sAPI.length() > 3)
          sTitle1 = new String(  this.stHeader.sName + " (" +
                                 this.stHeader.sAPI  + ")" );
        else
          sTitle1 = new String(  this.stHeader.sName );
//      }
//      else
//      {
//        sTitle1 = new String( this.stHeader.sName );
//      }

      if (txtName != null) txtName.setText(this.stHeader.sName);

      if ((this.stHeader.iTownship > 0) &&
          (this.stHeader.iRange > 0)    &&
          (this.stHeader.iSection > 0))
      {
        sTitle1 = new String(
                  sTitle1 +
                  " T: " + this.stHeader.iTownship + this.stHeader.sTownship +
                  " R: " + this.stHeader.iRange    + this.stHeader.sRange    +
                  " S: " + this.stHeader.iSection );
      }

      sTitle2 = new String("    ");
      if (this.stHeader.dLatitude != 0.0)
        sTitle2 = new String( sTitle2 + "Latitude: " +
                              this.stHeader.dLatitude + " " );

      if (this.stHeader.dLongitude != 0.0)
        sTitle2 = new String( sTitle2 + "Longitude: " +
                              this.stHeader.dLongitude + " " );

      if (this.stHeader.dGL > 0.0)
        sTitle2 = new String( sTitle2 + "Elevation (GL): " +
                              this.stHeader.dGL + " " );
      else if (this.stHeader.dKB > 0.0)
        sTitle2 = new String( sTitle2 + "Elevation (KB): " +
                              this.stHeader.dKB + " " );

      if (this.stHeader.depth > 0.0)
      {
//        if (stStruct.iDataType == iqstrat.iqstratStruct._WELL_DATA)
          sTitle2 = new String(sTitle2 + "Depth: " + this.stHeader.depth + " ");
//        else
//          sTitle2 = new String(sTitle2 + "Height: " + this.stHeader.depth + " ");
      }
    }

    if (plot != null) plot.setTitles(sTitle1, sTitle2);
  }

  /** Method setTitles( String sTitle1, String sTitle2 )
   * <p> This method will set the titles for this plot
   * @param sTitle1 = The first title
   * @param sTitle2 = The second title
   */

  public void setTitles( String sTitle1, String sTitle2 )
  {
    if (plot != null) { plot.setTitles(sTitle1, sTitle2); }
  }

  /** Method setData( seisStruct st )
   * <p> This method will set the seismic Data Structure
   * @param iData - Data being passed
   * @param st - seismic Data Structure
   */

  public void setLimits(int iData, double dMinimum, double dMaximum)
  {
	if (plot != null) { plot.setLimits( iData, dMinimum, dMaximum); }
  }

  /** Method setData( int iData, seisStruct st )
   * <p> This method will set the Seismic Data Structure
   * @param iData - Data being passed
   * @param st    - Seismic Data Structure
   */

  public void setData( int iData, seisStruct st )
  {
    if (plot != null) plot.setData( iData, st );
  }

  /** Method setBaro( int iData, seisStruct st )
   * <p> This method will set the Seismic Data Structure
   * @param iData - Data being passed
   * @param st    - Seismic Data Structure
   */

  public void setBaro( int iData, seisStruct st )
  {
    if (plot != null) plot.setBaro( iData, st );
  }

  /** Method setTidal()
   *  <p> This method will set the Tidal Parameters Data Structure
   *  @param st = the Tidal Parameters Data Structure
   *

  public void setTidal(psiTidalStruct st)
  {
    if (plot != null) { plot.setTidal(st); }
  }
  */
}

/*
 *  @version 1.1 03/05/2015
 *  @author  John Victorine
 */
