/*
 * @seisPlot.java Version 1.1 03/05/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package seismic.plot;

import java.sql.Timestamp;
import java.sql.Date;
import java.text.SimpleDateFormat;
import java.text.ParseException;

import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.awt.image.*;
import java.util.Observable;
import javax.swing.*;

import cmn.cmnStruct;              // Global Common Data Structure
import seismic.seisStruct;         // Seismic Data Structure
import seismic.plot.seisPlotTrack; // Seismic Data Plot Track

//import psi.psiTidalStruct;         // Tidal Parameters Data Structure

/** CLASS seisPlot
 *  <p> This Class will create the Seismic Data Plot.
 *
 *  @version 1.1 03/05/2015
 *  @author  John R. Victorine
 */

public class seisPlot extends Canvas
    implements MouseListener, MouseMotionListener
{
  private static final int _SEISMIC       = seismic.seisTracksStruct._SEISMIC;
  private static final int _SEISMIC_AVG   = seismic.seisTracksStruct._SEISMIC_AVG;
  private static final int _SEISMIC_MAG   = seismic.seisTracksStruct._SEISMIC_MAG;
  private static final int _PRESSURE_TEST = seismic.seisTracksStruct._PRESSURE_TEST;
  private static final int _PRESSURE_AVG  = seismic.seisTracksStruct._PRESSURE_AVG;
  private static final int _PRESSURE_MOD  = seismic.seisTracksStruct._PRESSURE_MOD;
  private static final int _PRESSURE_BARO = seismic.seisTracksStruct._PRESSURE_BARO;
  private static final int _PSI_WAVELETS  = seismic.seisTracksStruct._PSI_WAVELETS;

  public static final int _SOURCE      = seismic.seisTracksStruct._SOURCE;
  public static final int _TYPE        = seismic.seisTracksStruct._TYPE;
  public static final int _DESCRIPTION = seismic.seisTracksStruct._DESCRIPTION;
  public static final int _STATUS      = seismic.seisTracksStruct._STATUS;

  // Input Variables

  private cmnStruct  stCMN        = null; // Global Common Data Structure
  private Observable notifier     = null; // Observable

  private Timestamp  tsStart      = null; // Starting Date
  private Timestamp  tsEnd        = null; // Ending Date
  private int        iScale       = -1;   // Plot Scale
  private int        iSelected[]  = null; // Selected Tracks

  private int        iLegend      = 0;

  private Cursor     normalCursor = null; // Normal Cursor
  private Cursor     mouseCursor  = null; // Edit Track Cursor

  // Global Variables

  private int        iStart[]     = null; // Array of Track Starting X-Position
  private int        iWide[]      = null; // Array of Track Widths
  private int        iButtons     = 0;    // Total Number of Tracks

  private JTextField txtTime      = null; // Time Textfield

//  private seisStruct st           = null; // Seismic Data Structure
//  private seisStruct stAvg        = null; // Seismic Data Structure
//  private seisStruct stMag        = null; // Seismic Data Structure
//  private seisStruct stPSI        = null; // Seismic Data Structure - Pressure Data
//  private seisStruct stPSIAvg     = null; // Seismic Data Structure - Pressure Data

  // Plot Tracks

  private seisPlotTrack pSeismic     = null; // Seismic Plot Track
  private seisPlotTrack pSeisAvg     = null; // Averaged Seismic Data Plot Track
  private seisPlotTrack pSeisMag     = null; // Seismic Data Envelope Plot Track
  private seisPlotTrack pSeisPSI     = null; // Pressure Data Envelope Plot Track
  private seisPlotTrack pSeisPSIAvg  = null; // Pressure Data Envelope Plot Track
  private seisPlotTrack pSeisPSIMod  = null; // Pressure Data Envelope Plot Track
  private seisPlotTrack pSeisPSIBaro = null; // Pressure Data Envelope Plot Track

//  private psiTidalStruct stTidal     = null; // Tidal Data Structure

  private int           iDate        = 20;
  private int           iDateTotal   = 0;
  private String        sDate[]      =
  { "", "", "", "", "", "", "", "", "", "",
    "", "", "", "", "", "", "", "", "", "" };

  // Plot Title

  private String sTitle1 = "";
  private String sTitle2 = "";

  // Plot Variables

  private int    iWidth          = 650; // Width of Plot
  private int    iHeight         = 100; // Height of Plot with the Titles
  private int    iLogHeight      = 100; // Height of Plot without the Titles
  private int    iIncrementY     = 100; // Increment Height

  private double dIncrementTime  = 0.0; // Increment Time
  private int    iStartTimeLabel = 10;  // Start Time Label at

  private int    iXBegin         = 0;
  private int    iXEnd           = 0;
  private int    iYBegin         = 0;
  private int    iYEnd           = 0;

  public static final int PLOT_TITLES = seismic.seisTracksStruct.PLOT_TITLES;
  public static final int TIME_TRACK  = seismic.seisTracksStruct.TIME_TRACK;
  public static final int LABELSTART  = seismic.seisTracksStruct.LABELSTART;

  // Color Variables

  public static final Color cRED     = new Color( 180,   0,   0 );
  public static final Color cGREEN   = new Color(   0, 180,   0 );
  public static final Color cBLUE    = new Color(   0,   0, 180 );
  public static final Color cCYAN    = new Color(   0, 180, 180 );
  public static final Color cMAGENTA = new Color( 180,   0, 180 );
  public static final Color cYELLOW  = new Color( 180, 180,   0 );
  public static final Color cORANGE  = new Color( 204, 102,   0 );
  public static final Color cBROWN   = new Color( 102, 102,   0 );

  public static final Color COLORS[] =  // Color for Log Curves
      { cRED, cGREEN, cBLUE, cBROWN, cMAGENTA, cORANGE, cYELLOW, cCYAN };

  /** CONSTRUCTOR seisPlot()
   *  <p> This is the constructor for this class.
   *  @param stCMN      = Common Data Structure
   *  @param tsStart    = The starting date
   *  @param tsEnd      = The ending date
   *  @param iScale     = The time scale
   *  @param iSelected  = The selected tracks for plotting
   */

  public seisPlot( cmnStruct  stCMN,
                   Timestamp  tsStart,
                   Timestamp  tsEnd,
                   int        iScale,
                   int        iSelected[] )
  {
    this.stCMN = stCMN;

    setPlotWidth(iSelected);
    setPlotHeight(iScale, tsEnd, tsStart);

    pSeismic     = new seisPlotTrack( tsStart, tsEnd, iScale );
    pSeisAvg     = new seisPlotTrack( tsStart, tsEnd, iScale );
    pSeisMag     = new seisPlotTrack( tsStart, tsEnd, iScale );
    pSeisPSI     = new seisPlotTrack( tsStart, tsEnd, iScale );
    pSeisPSIAvg  = new seisPlotTrack( tsStart, tsEnd, iScale );
    pSeisPSIMod  = new seisPlotTrack( tsStart, tsEnd, iScale );
    pSeisPSIBaro = new seisPlotTrack( tsStart, tsEnd, iScale );

    addMouseListener(this);
    addMouseMotionListener(this);

    this.setBackground(Color.white);
  }

  /** CONSTRUCTOR seisPlot()
   *  <p> This is the constructor for this class.
   *  @param stCMN        = Common Data Structure
   *  @param notifier     = the observable object
   *  @param tsStart      = The starting date
   *  @param tsEnd        = The ending date
   *  @param iScale       = The time scale
   *  @param iSelected    = The selected tracks for plotting
   *  @param normalCursor = Normal Cursor
   *  @param mouseCursor  = Hand Cursor
   */

  public seisPlot( cmnStruct  stCMN,
                   Timestamp  tsStart,
                   Timestamp  tsEnd,
                   int        iScale,
                   int        iSelected[],
                   Cursor     normalCursor,
                   Cursor     mouseCursor )
  {
	this.stCMN    = stCMN;

    this.normalCursor = normalCursor;
    this.mouseCursor  = mouseCursor;

    setPlotWidth(iSelected);
    setPlotHeight(iScale, tsEnd, tsStart);

    pSeismic     = new seisPlotTrack( tsStart, tsEnd, iScale );
    pSeisAvg     = new seisPlotTrack( tsStart, tsEnd, iScale );
    pSeisMag     = new seisPlotTrack( tsStart, tsEnd, iScale );
    pSeisPSI     = new seisPlotTrack( tsStart, tsEnd, iScale );
    pSeisPSIAvg  = new seisPlotTrack( tsStart, tsEnd, iScale );
    pSeisPSIMod  = new seisPlotTrack( tsStart, tsEnd, iScale );
    pSeisPSIBaro = new seisPlotTrack( tsStart, tsEnd, iScale );

    addMouseListener(this);
    addMouseMotionListener(this);

    this.setBackground(Color.white);
  }

  /** Method close()
   * <p> This method will set object to null to force JAVA to reallocate memory
   */

  public void close()
  {
    // Input Variables

    stCMN        = null; // Global Common Data Structure
    notifier     = null; // Observable

    tsStart      = null;
    tsEnd        = null;

    iSelected    = null;

    normalCursor = null;
    mouseCursor  = null;

    // Global Variables

    iStart       = null;
    iWide        = null;

    txtTime      = null;  // Time Textfield

//    st           = null;
//    stAvg        = null;
//    stMag        = null;
//    stPSI        = null;
//    stPSIAvg     = null;

    if (pSeismic != null)
      pSeismic.close();
    pSeismic     = null;

    if (pSeisAvg != null)
      pSeisAvg.close();
    pSeisAvg     = null;

    if (pSeisMag != null)
      pSeisMag.close();
    pSeisMag     = null;

    if (pSeisPSI != null)
      pSeisPSI.close();
    pSeisPSI     = null;

    if (pSeisPSIAvg != null)
      pSeisPSIAvg.close();
    pSeisPSIAvg  = null;

    if (pSeisPSIMod != null)
      pSeisPSIMod.close();
    pSeisPSIMod  = null;

    if (pSeisPSIBaro != null)
      pSeisPSIBaro.close();
    pSeisPSIBaro  = null;

//    stTidal = null;
  }

  /** Method refresh()
   *  <p> This method will refresh the plot
   */

  public void refresh() { repaint(); }

  /* =============================================================== *
   * ------------------------- GET METHODS ------------------------- *
   * =============================================================== */

  /** Method getPlotWidth()
   * <p> This method will return the Plot Width
   * @return iWidth = The Plot Width
   */

  public int getPlotWidth()  { return (iWidth); }

  /** Method getPlotHeight()
   * <p> This method will return the Plot Height
   * @return iHeight = The Plot Height
   */

  public int getPlotHeight() { return (iHeight); }

  /** Method getImage()
   * <p> This method will convert this plot to a buffered image
   * @return image = profile plot buffered image
   */

  public BufferedImage getImage()
  {
    BufferedImage image = null;
    int w = this.getWidth();
    int h = this.getHeight();

    image = new BufferedImage( w, h, BufferedImage.TYPE_INT_RGB );

    Graphics2D g2 = image.createGraphics();
    g2.setColor( Color.white );
    g2.fillRect( 0, 0, w, h );
    this.paint(g2);
    g2.dispose();

    return (image);
  }

  /* =============================================================== *
   * ------------------------- SET METHODS ------------------------- *
   * =============================================================== */

  /** Method setObservable(Observable notifier)
   * <p> This method will set the Observerable Object
   * @param notifier = the observable object
   */

  public void setObservable(Observable notifier) { this.notifier = notifier; }

  /** Method setSelected()
   * <p> This method will change the selected tracks that are plotted
   *  @param iSelected  = The Tracks that will be displayed
   */

  public void setSelected(int iSelected[])
  {
    setPlotWidth(iSelected);
    repaint();
  }

  /** Method setTextfield(JTextField txtTime)
   * <p> This method will set the depth textfield
   * @param txtTime = the depth text field for displaying the cursor location
   */

  public void setTextfield(JTextField txtTime) { this.txtTime = txtTime; }

  /** Method setPlotWidth()
   * <p> This method will set the Plot Width
   *  @param iSelected  = The Tracks that will be displayed
   */

  public void setPlotWidth(int iSelected[])
  {
    int iTrackWidth = 0;

    iWidth = TIME_TRACK;

    this.iSelected = iSelected;
    if (iSelected != null)
      this.iButtons = iSelected.length;

    if (this.iButtons > 0)
    {
      iStart = new int[this.iButtons];
      iWide  = new int[this.iButtons];

      for (int i=0; i<this.iButtons; i++)
      {
        iTrackWidth = 0;
        for (int j=0; j<seismic.seisTracksStruct.TOTAL_TYPES; j++)
        {
          if (iSelected[i] != -1)
          {
            if (seismic.seisTracksStruct.TRACK[iSelected[i]][_TYPE].equals(
                seismic.seisTracksStruct.TYPE[j][0]))
            {
              iTrackWidth = cmn.cmnString.stringToInt(
                              seismic.seisTracksStruct.TYPE[j][2]);
		    }
		  }
		}

        iStart[i] = iWidth;
        iWide[i]  = iTrackWidth;
        iWidth    = iWidth + iTrackWidth;
	  }
	}

	iLegend = iWidth;
	iWidth  = iWidth+200;
  }

  /** Method setPlotHeight()
   *  <p> This method will set the Plot Height.
   * @param iScale    = The Time Track Scale identifier
   * @param tsMaximum = The Maximum Date
   * @param tsMinimum = The Minimum Date
   */

  public void setPlotHeight(int iScale, Timestamp tsMaximum, Timestamp tsMinimum)
  {
    this.tsStart = new Timestamp(tsMinimum.getTime());
    this.tsEnd   = new Timestamp(tsMaximum.getTime());
    this.iScale  = iScale;

    long lstart  = tsMinimum.getTime();
    long lend    = tsMaximum.getTime();
    long diff    = lend-lstart;

    iLogHeight = (int) ((100 * (double) diff) /
                   seismic.seisTracksStruct.SCALE[iScale]);

    iHeight    = PLOT_TITLES + iLogHeight;
    dIncrementTime = seismic.seisTracksStruct.SCALE[iScale];
  }

  /** Method setScale(int iScale, double dMaximum, double dMinimum)
   * <p> This method will set the Plot scale in the
   *     Cross Plot Data List Structure
   * @param iScale    = The Time Track Scale identifier
   * @param tsMaximum = The Maximum Timestamp
   * @param tsMinimum = The Minimum Timestamp
   */

  public void setScale(int iScale, Timestamp tsMaximum, Timestamp tsMinimum)
  {
    setPlotHeight(iScale, tsMaximum, tsMinimum);

    if (pSeismic != null) { pSeismic.setPlotHeight( iScale, tsMaximum, tsMinimum ); }
    if (pSeisAvg != null) { pSeisAvg.setPlotHeight( iScale, tsMaximum, tsMinimum ); }
    if (pSeisMag != null) { pSeisMag.setPlotHeight( iScale, tsMaximum, tsMinimum ); }
    if (pSeisPSI != null) { pSeisPSI.setPlotHeight( iScale, tsMaximum, tsMinimum ); }
    if (pSeisPSIAvg != null) { pSeisPSIAvg.setPlotHeight( iScale, tsMaximum, tsMinimum ); }
    if (pSeisPSIMod != null) { pSeisPSIMod.setPlotHeight( iScale, tsMaximum, tsMinimum ); }
    if (pSeisPSIBaro != null) { pSeisPSIBaro.setPlotHeight( iScale, tsMaximum, tsMinimum ); }

    repaint();
  }

  /** Method setTitles()
   * <p> This method will set the titles for this plot
   * @param sTitle1 = The first title
   * @param sTitle2 = The second title
   */

  public void setTitles(String sTitle1, String sTitle2)
  {
    this.sTitle1 = new String(sTitle1);
    this.sTitle2 = new String(sTitle2);

    repaint();
  }

  /** Method setData( seisStruct st )
   * <p> This method will set the seismic Data Structure
   * @param iData - Data being passed
   * @param st - seismic Data Structure
   */

  public void setLimits(int iData, double dMinimum, double dMaximum)
  {
	switch (iData)
	{
	  case _SEISMIC:
  	    if (pSeismic != null) { pSeismic.setLimits( dMinimum, dMaximum); }
	    break;

	  case _SEISMIC_AVG:
	    if (pSeisAvg != null) { pSeisAvg.setLimits( dMinimum, dMaximum); }
	    break;

	  case _SEISMIC_MAG:
	    if (pSeisMag != null) { pSeisMag.setLimits( dMinimum, dMaximum); }
	    break;

	  case _PRESSURE_TEST:
	    if (pSeisPSI != null) { pSeisPSI.setLimits( dMinimum, dMaximum); }
	    break;

	  case _PRESSURE_AVG:
	  case _PRESSURE_MOD:
	    if (pSeisPSIAvg != null) { pSeisPSIAvg.setLimits( dMinimum, dMaximum); }
	    if (pSeisPSIMod != null) { pSeisPSIMod.setLimits( dMinimum, dMaximum); }
	    break;

	  case _PRESSURE_BARO:
	    if (pSeisPSIBaro != null) { pSeisPSIBaro.setLimits( dMinimum, dMaximum); }
	    break;
    }

    repaint();
  }

  /** Method setData(int iData, seisStruct st)
   * <p> This method will set the Seismic Data Structure
   * @param iData - Data being passed
   * @param st - Seismic Data Structure
   */

  public void setData(int iData, seisStruct st)
  {
    String tokens[]    = null;
    String sDelimiter  = new String("[ ]+");
    String str         = "";
    String strdate     = "";
    int    icnt        = 0;
    String sdate       = "";
	long       lstart = (long) 0;
	long       lend   = (long) 0;
	long       lday   = (long) 86400000;
	long       ldays  = (long) 0;
	int        iTotal = 0;

	switch (iData)
	{
	  case _SEISMIC:
	    if (pSeismic != null) { pSeismic.setData(st); } //this.st); }
	    break;

	  case _SEISMIC_AVG:
	    if (pSeisAvg != null) { pSeisAvg.setData(st); } //this.stAvg); }
	    break;

	  case _SEISMIC_MAG:
	    if (pSeisMag != null) { pSeisMag.setData(st); } //this.stMag); }
	    break;

	  case _PRESSURE_TEST:
	    if (st != null)
	    {
		  lstart  = tsStart.getTime();
		  lend    = tsEnd.getTime();

          ldays   = (lend - lstart) / lday;
          iTotal  = (int) ldays;

          iDate   = iTotal;
          sDate   = new String[iTotal];

		  iDateTotal = 0;
		  for (int i=0; i<st.iRows; i++)
		  {
            str    = new String(st.ts[i].toString());
	        tokens = str.split( sDelimiter );

	        if (tokens.length > 1)
	        {
	          strdate = new String(tokens[0].trim());

	          if ((iDateTotal < iDate) && (!strdate.equals(sdate)))
	          {
				sDate[iDateTotal] = new String(strdate);
				sdate             = new String(strdate);
				iDateTotal++;
			  }
			}
		  }
		}

	    if (pSeisPSI != null) { pSeisPSI.setData(st); } //this.stPSI); }
	    break;

	  case _PRESSURE_AVG:
	    if (pSeisPSIAvg != null) { pSeisPSIAvg.setData(st); } //this.stPSIAvg); }
	    break;

	  case _PRESSURE_MOD:
	    if (pSeisPSIMod != null) { pSeisPSIMod.setData(st); } //this.stPSIAvg); }
	    break;

	  case _PRESSURE_BARO:
	    if (pSeisPSIBaro != null) { pSeisPSIBaro.setData(st); } //this.stPSIAvg); }
	    break;
    }

    repaint();
  }

  /** Method setBaro(int iData, seisStruct st)
   * <p> This method will set the Seismic Data Structure
   * @param iData - Data being passed
   * @param st - Seismic Data Structure
   */

  public void setBaro(int iData, seisStruct st)
  {
	switch (iData)
	{
	  case _PRESSURE_MOD:
	    if (pSeisPSIMod != null) { pSeisPSIMod.setBaro(st); } //this.stPSIAvg); }
	    break;
    }
  }

  /** Method setTidal()
   *  <p> This method will set the Tidal Parameters Data Structure
   *  @param st = the Tidal Parameters Data Structure
   *

  public void setTidal(psiTidalStruct st)
  {
	this.stTidal = st;

    if (pSeisPSIAvg != null) { pSeisPSIAvg.setTidal(st); }
    if (pSeisPSIMod != null) { pSeisPSIMod.setTidal(st); }
  }
*/
  /* =============================================================== *
   * --------------------------- COMPUTE --------------------------- *
   * =============================================================== */

  /** Method computeMouseDepth(int iYm)
   * <p> This method will compute the Mouse Depth
   * @param iYm = the cursor y-coordinate
   * @return depth = depth computed from mouse cursor location
   */

  public double computeMouseDepth(int iYm)
  {
    double depth = 0.0;

    if (iYm > PLOT_TITLES)
    {
//      depth = depthStart + (depthEnd-depthStart) * (iYm-PLOT_TITLES)/iLogHeight;

//      if (txtDepth != null)
//      {
//        depth = (double) (Math.round(depth * 100.0) / 100.0);
//        txtDepth.setText(""+depth);
//      }
    }

    return (depth);
  }

  /* =============================================================== *
   * -------------------- MouseListener Methods -------------------- *
   * =============================================================== */

  /** Method mouseMoved()
   *  <p> This method will retrieve the location of the mouse pointer and
   *      convert the postion to depth position.
   * @param event - Mouse Moved Event
   */

  public void mouseMoved(MouseEvent event)
  {
    int    iXm  = event.getX();
    int    iYm  = event.getY();
    double depth = 0.0;

    if (iYm > PLOT_TITLES)
    {
      depth = computeMouseDepth(iYm);

//      if (txtTime != null)
//      {
//        txtTime.setText(""+depth);
//      }
    }

  }

  public void mouseEntered(MouseEvent event) { }
  public void mouseExited(MouseEvent event)  { }

  /** Method mousePressed(MouseEvent)
   *  <p> gets the current coordinates and creates a new rectangle,
   *      updates the rectangle to be drawn and
   *      calls repaint on the parent panel
   *  @param event - MouseEvent info
   */

  public void mousePressed(MouseEvent event)
  {
    iXBegin      = event.getX();
    iXEnd        = 0;
    iYBegin      = event.getY();
    iYEnd        = 0;
  }

  /** Method mouseDragged(MouseEvent)
   *  <p> updates the size of the rectangle as the user drags the mouse
   *  @param event - MouseEvent info
   */

  public void mouseDragged(MouseEvent event)
  {
  }

  /** Method mouseReleased(MouseEvent)
   *  <p> updates the size of the rectangle as the user releases the mouse
   *  @param event - MouseEvent info
   */

  public void mouseReleased(MouseEvent event)
  {
  }

  /** Method mouseClicked()
   *  <p> Mouse Clicked Method retrieve the location of the mouse pointer
   *      location and decides if the it was clicked in on a Core Button
   *      then it will display a core image in a browser or did it click on
   *      the Well Profile Plot Region then display the Formation Tops &
   *      PfEFFER Region Selection Frame and pass back to the Formation Tops
   *      & PfEFFER Region Selection Frame the Depth of the Mouse Click.
   * @param event - Mouse Clicked Event
   */

  public void mouseClicked(MouseEvent event)
  {
    int    iXm  = event.getX();
    int    iYm  = event.getY();
  }

  /* =============================================================== *
   * ------------------------ DRAW METHODS ------------------------- *
   * =============================================================== */

  /** Method drawTitles(Graphics g)
   * <p> This method will draw the Titles for this plot
   * @param g - Graphics Parameter.
   */

  public void drawTitles(Graphics g)
  {
    Font        fsb    = new Font("Serif", Font.BOLD, 12);
    FontMetrics fsbm   = g.getFontMetrics(fsb);

    g.setFont( fsb );
    g.setColor(Color.black);

    g.drawString(sTitle1, 70, 10);
    g.drawString(sTitle2, 70, 24);
  }

  /** Method drawDateLabel(Graphics g)
   *  <p> This method will draw the date scale
   *  @param g - Graphics Parameter.
   */

  public void drawDateLabel(Graphics g)
  {
    SimpleDateFormat dfm  = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSSSSS");

    Font             fsb  = new Font("Serif", Font.BOLD, 14);
    FontMetrics      fsbm = g.getFontMetrics(fsb);

    String sText  = "";
    double dRatio = 0.0;

    long   lstart = tsStart.getTime();
    long   lend   = tsEnd.getTime();
    long   ltime  = (long) 0;

    int    iStartTrack = 0;
    int    iWidthTrack = TIME_TRACK;
    int    iY1         = 0;

    g.setFont( fsb );
    g.setColor(Color.blue);

    for (int i=0; i<iDateTotal; i++)
    {
	  sText = new String(sDate[i] + " 00:00:00.0");

      try
      {
        ltime   = dfm.parse(sText).getTime();
//        tsStart = new Timestamp( ltime );
	  }
      catch (ParseException e)
      {
        e.printStackTrace();
      }

      dRatio = (double) iLogHeight * (ltime - lstart) / (lend - lstart);
      iY1    = PLOT_TITLES + (int) dRatio;

      if ((iY1 >= PLOT_TITLES)  && (iY1 < iHeight))
      {
        g.drawLine(iStartTrack, iY1, iStartTrack+iWidthTrack, iY1);
        g.drawString(sDate[i], iStartTrack, iY1); //iStartTimeLabel, iY1);
	  }
	}
  }

  /** Method drawTime(Graphics g)
   * <p> This method will draw the time scale
   * @param g - Graphics Parameter.
   */

  public void drawTime(Graphics g)
  {
    int    i=0;
    int    j=0;
    int    iDepth      = 0;
    int    jXinc       = 0;
    int    iStartTrack = 0;
    int    iWidthTrack = TIME_TRACK;
    int    iY1         = 0;
    int    iY2         = 0;
    long   ltime       = (long) 0;
    long   lstart      = (long) 0;

    String tokens[]    = null;
    String sDelimiter  = new String("[ ]+");
    String str         = "";
    String sDate       = "";
    String sTime       = "";
    Timestamp tsTime   = null;

    Font fsb = new Font("Serif", Font.BOLD, 12);
    FontMetrics fsbm = g.getFontMetrics(fsb);

    g.setFont( fsb );
    g.setColor(Color.black);

    lstart = tsStart.getTime();

	str = new String(tsStart.toString());
	tokens = str.split( sDelimiter );
	if (tokens.length > 0)
	  sDate = new String(tokens[0].trim());

    for (i=0; i<=iLogHeight; i+=iIncrementY)
    {
      ltime = (long) (lstart +
                      (long)(i * dIncrementTime / (double) iIncrementY));
      tsTime = new Timestamp( ltime );

	  str = new String(tsTime.toString());
	  tokens = str.split( sDelimiter );
	  if (tokens.length > 0)
	    sTime = new String(tokens[1].trim());

      iY1 = i+PLOT_TITLES;

//      jXinc = iIncrementY / 5;
//      for (j=0; j<5; j++)
      jXinc = iIncrementY / 6;
      for (j=0; j<6; j++)
      {
        iY2 = i+(j*jXinc)+PLOT_TITLES;

        if ((iY2 >= PLOT_TITLES)  && (iY2 < iHeight))
        {
          g.setColor(Color.lightGray);
          g.drawLine(iStartTrack, iY2, iStartTrack + iWidthTrack, iY2);
        }
      }

      g.setColor(Color.black);
      g.drawLine(iStartTrack, iY1, iStartTrack+iWidthTrack, iY1);

      if (iScale < seismic.seisTracksStruct.SCALE_1440)
        g.drawString(sTime, iStartTimeLabel, iY1);
    }

    g.drawLine(iStartTrack, PLOT_TITLES, iStartTrack+iWidthTrack, PLOT_TITLES);
  }

  /** Method drawTrackLabels()
   *  <p> This method will draw the labels on the LAS Plot.
   *   @param g - Graphics Parameter.
   */

  public void drawTrackLabels(Graphics g)
  {
    int length      = 0;
    int iStartTrack = 0;
    int iWidthTrack = TIME_TRACK;
    String sTemp    = "";

    g.setColor(Color.black);

    Font fsb = new Font("Serif", Font.BOLD, 12);
    FontMetrics fsbm = g.getFontMetrics(fsb);
    g.setFont( fsb );

    length = sTemp.length();
    g.drawString(sTemp, iWidth/2-length*4, 14);
    g.drawLine(iStartTrack,
               LABELSTART,
               iWidthTrack,
               LABELSTART);

    g.setColor(Color.black);
    g.drawString("Time (CDT)", iStartTimeLabel, LABELSTART + 10);
  }

  /** Method drawLegend()
   *  <p> This method will draw the Legend
   * @param g - Graphics Parameter.
   *

  public void drawLegend(Graphics g)
  {
	double      dF     = 0.0;
	double      dTemp  = 0.0;
    Font        fsb    = new Font("Serif", Font.BOLD, 12);
    FontMetrics fsbm   = g.getFontMetrics(fsb);
    int         iPos   = PLOT_TITLES;

    g.setFont( fsb );
    g.setColor(Color.black);

	if (stTidal != null)
	{
      g.drawString("Moon Latitude: " + stTidal.dMoon, iLegend+5, iPos);
      iPos = iPos + 15;
      g.drawString("Porosity:      " + stTidal.dPHI, iLegend+5, iPos);
      iPos = iPos + 15;
      g.drawString("Cw:            " + stTidal.dCw + " [1/GPa]", iLegend+5, iPos);
      iPos = iPos + 30;

      g.drawString("Diurnal: ", iLegend+5, iPos);
      iPos = iPos + 15;

	  dTemp = (1.0/stTidal.dFd)/3600.0;
	  dF   = (double) ((int) (dTemp * 10000.0) / 10000.0);

      g.drawString("Frequency: " + dF + " [hrs/cycle]", iLegend+25, iPos);
      iPos = iPos + 15;
      g.drawString("Phase:     " + stTidal.dPHId, iLegend+25, iPos);
      iPos = iPos + 30;

      g.drawString("Semi-diurnal: ", iLegend+5, iPos);
      iPos = iPos + 15;

	  dTemp = (1.0/stTidal.dFs)/3600.0;
	  dF   = (double) ((int) (dTemp * 10000.0) / 10000.0);

      g.drawString("Frequency: " + dF + " [hrs/cycle]", iLegend+25, iPos);
      iPos = iPos + 15;
      g.drawString("Phase:     " + stTidal.dPHIs, iLegend+25, iPos);
      iPos = iPos + 30;

      g.drawString("Filtered Pressure Track", iLegend+5, iPos);
      iPos = iPos + 15;

      g.setColor(COLORS[0]);
      g.drawLine(iLegend+5, iPos, iLegend+20, iPos);
      g.drawString("Pressure Slope [psi]: " +
                   stTidal.dStart + " to " + stTidal.dEnd, iLegend+25, iPos);
      iPos = iPos + 15;

      g.setColor(COLORS[2]);
      g.drawLine(iLegend+5, iPos, iLegend+20, iPos);
      g.drawString("Pressure Slope + Tidal Pressure [psi]: ", iLegend+25, iPos);
      iPos = iPos + 15;

      g.setColor(COLORS[1]);
      g.drawLine(iLegend+5, iPos, iLegend+20, iPos);
      g.drawString("Filtered Pressure [psi]: ", iLegend+25, iPos);
      iPos = iPos + 15;
	}
  }
*/
  /** Method drawLegend()
   *  <p> This method will draw the Legend
   * @param g - Graphics Parameter.
   *

  public void drawLegend(Graphics g)
  {
	double      dF     = 0.0;
	double      dTemp  = 0.0;
    Font        fsb    = new Font("Serif", Font.BOLD, 12);
    FontMetrics fsbm   = g.getFontMetrics(fsb);
    int         iPos   = PLOT_TITLES;

    g.setFont( fsb );
    g.setColor(Color.black);

	if (stTidal != null)
	{
//      g.drawString("Moon Latitude: " + stTidal.dMoon, iLegend+5, iPos);
//      iPos = iPos + 15;
      g.drawString("Porosity:      " + stTidal.dPHI, iLegend+5, iPos);
      iPos = iPos + 15;
      g.drawString("Cw:            " + stTidal.dCw + " [1/GPa]", iLegend+5, iPos);
      iPos = iPos + 30;

      g.drawString("Correction: ", iLegend+5, iPos);
      iPos = iPos + 15;
      g.drawString("Phase:     " + stTidal.dPHId, iLegend+25, iPos);
      iPos = iPos + 30;

      g.drawString("Filtered Pressure Track", iLegend+5, iPos);
      iPos = iPos + 15;

      g.setColor(COLORS[0]);
      g.drawLine(iLegend+5, iPos, iLegend+20, iPos);
      g.drawString("Pressure Slope [psi]: " +
                   stTidal.dStart + " to " + stTidal.dEnd, iLegend+25, iPos);
      iPos = iPos + 15;

      g.setColor(COLORS[2]);
      g.drawLine(iLegend+5, iPos, iLegend+20, iPos);
      g.drawString("Pressure Slope + Tidal Pressure [psi]: ", iLegend+25, iPos);
      iPos = iPos + 15;

      g.setColor(COLORS[1]);
      g.drawLine(iLegend+5, iPos, iLegend+20, iPos);
      g.drawString("Filtered Pressure [psi]: ", iLegend+25, iPos);
      iPos = iPos + 30;

      g.setColor(Color.black);
      g.drawString("Filtered - Tidal Pressure Track", iLegend+5, iPos);
      iPos = iPos + 15;

      g.setColor(COLORS[0]);
      g.drawLine(iLegend+5, iPos, iLegend+20, iPos);
      g.drawString("Pressure Slope [psi]: " +
                   stTidal.dStart + " to " + stTidal.dEnd, iLegend+25, iPos);
      iPos = iPos + 15;

      g.setColor(COLORS[2]);
      g.drawLine(iLegend+5, iPos, iLegend+20, iPos);
      g.drawString("Filtered Pressure - Tidal Pressure [psi]: ", iLegend+25, iPos);
      iPos = iPos + 15;

      g.setColor(COLORS[7]);
      g.drawLine(iLegend+5, iPos, iLegend+20, iPos);
      g.drawString("Pressure with respect to Barometric Pressure [psi] ", iLegend+25, iPos);
      iPos = iPos + 15;

      g.setColor(COLORS[7]);
      g.drawString("  Average Barometric Pressure = " + stTidal.dPHIs + " [psi]", iLegend+25, iPos);
	}
  }
*/
  /** Method draw(graphics g)
   * <p> This method will draw the stratigraphic plot
   * @param g - Graphics Parameter.
   */

  public void draw(Graphics g)
  {
    int iStartWidth = TIME_TRACK;
    int iTrackWidth = 0;

    g.setColor(Color.white);
    g.fillRect(0, 0, iWidth, iHeight);

    drawTitles(g);
    drawTrackLabels(g);
    drawTime(g);
    drawDateLabel(g);
//    drawLegend(g);

//    if (stCMN != null)
//      g.drawImage( stCMN.pIcon2, 0, 0, 58, 58, this );

    for (int i=0; i<iButtons; i++)
    {
      if (iSelected[i] != -1)
      {
		switch (iSelected[i])
		{
		  case _SEISMIC:
            pSeismic.draw(g, iSelected[i], iStart[i], iWide[i]);
		    break;

		  case _SEISMIC_AVG:
            pSeisAvg.draw(g, iSelected[i], iStart[i], iWide[i]);
		    break;

		  case _SEISMIC_MAG:
            pSeisMag.draw(g, iSelected[i], iStart[i], iWide[i]);
		    break;

		  case _PRESSURE_TEST:
            pSeisPSI.draw(g, iSelected[i], iStart[i], iWide[i]);
		    break;

		  case _PRESSURE_AVG:
            pSeisPSIAvg.draw(g, iSelected[i], iStart[i], iWide[i]);
		    break;

		  case _PRESSURE_MOD:
            pSeisPSIMod.draw(g, iSelected[i], iStart[i], iWide[i]);
		    break;

		  case _PRESSURE_BARO:
            pSeisPSIBaro.draw(g, iSelected[i], iStart[i], iWide[i]);
		    break;
		}
	  }
	}
  }

  /** Method paint(Graphics g)
   * <p> This method will paint stratigraphic Plot
   * @param g - Graphics Parameter.
   */

  public void paint(Graphics g)
  {
	draw(g);
  }
}

/*
 *  @version 1.1 03/05/2015
 *  @author  John Victorine
 */
