 /*
 * @seisPlotTrack.java Version 1.1 03/04/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package seismic.plot;

import java.awt.*;
import java.awt.geom.*;

import java.sql.Timestamp;
//import java.sql.Date;
//import java.text.SimpleDateFormat;
//import java.text.ParseException;

import seismic.seisStruct;

import psi.psiTidalStruct;         // Tidal Parameters Data Structure

/** CLASS seisPlotTrack
 *  <p> This Class will create the seismic track.
 *
 *  @version 1.1 03/04/2015
 *  @author  John R. Victorine
 */

public class seisPlotTrack extends Canvas
{
  private static final int _MEAN   = psi.psiTidalStruct._MEAN;
  private static final int _LINEAR = psi.psiTidalStruct._LINEAR;

  private static final double PSI = .00014504; // 1Pa = 145.04 X 10^-6 psi
  private static final double RHO = 1100.0;
  // Input Variables

  private int       iScale  = -1;     // Time Scale minutes / inch
  private Timestamp tsStart = null;   // Starting Date
  private Timestamp tsEnd   = null;   // Ending Date

  // Global Variables

  public static final Color cRED     = new Color( 180,   0,   0 );
  public static final Color cGREEN   = new Color(   0, 180,   0 );
  public static final Color cBLUE    = new Color(   0,   0, 180 );
  public static final Color cCYAN    = new Color(   0, 180, 180 );
  public static final Color cMAGENTA = new Color( 180,   0, 180 );
  public static final Color cYELLOW  = new Color( 180, 180,   0 );
  public static final Color cORANGE  = new Color( 204, 102,   0 );
  public static final Color cBROWN   = new Color( 102, 102,   0 );

  public static final Color COLORS[] =  // Color for Log Curves
      { cRED, cGREEN, cBLUE, cBROWN, cMAGENTA, cORANGE, cYELLOW, cCYAN };

  private seisStruct     st      = null; // seismic Data Structure
  private psiTidalStruct stTidal = null;
  private seisStruct     stMod   = null;
  private seisStruct     stBaro  = null;

/*
  private SimpleDateFormat dfm      = new SimpleDateFormat("MM-dd-yyyy HH:mm:ss");
  private double           dRA[]    = null;
  private double           dMoon[]  = null;
  private long             lMoon[]  = null;
*/

  // Plot Variables

  private int iWidth      = 100;  // Width of Plot
  private int iHeight     = 100;  // Height of Plot with the Titles
  private int iLogHeight  = 100;  // Height of Plot without the Titles
  private int iIncrementY = 100;  // Increment Height

  public static final int PLOT_TITLES = seismic.seisTracksStruct.PLOT_TITLES;
  public static final int LABELSTART  = seismic.seisTracksStruct.LABELSTART;

  /** CONSTRUCTOR seisPlotTrack()
   *  <p> This is the constructor for this class.
   *  @param tsStart  = The starting date
   *  @param tsEnd    = The ending date
   *  @param iScale   = The scale
   */

  public seisPlotTrack( Timestamp tsStart,
                        Timestamp tsEnd,
                        int       iScale )
  {
    setPlotHeight(iScale, tsEnd, tsStart);

    this.setBackground(Color.white);
  }

  /** Method close()
   * <p> This method will set object to null to force JAVA to reallocate memory
   */

  public void close()
  {
	st      = null;
	stTidal = null;
	stBaro  = null;

    if (stMod != null)
      stMod.delete();
    stMod = null;

	tsStart = null;
	tsEnd   = null;
  }

  /* =============================================================== *
   * ------------------------- GET METHODS ------------------------- *
   * =============================================================== */

  /** Method getPlotWidth()
   * <p> This method will return the Plot Width
   * @return iWidth = The Plot Width
   */

  public int getPlotWidth()  { return (iWidth); }

  /** Method getPlotHeight()
   * <p> This method will return the Plot Height
   * @return iHeight = The Plot Height
   */

  public int getPlotHeight() { return (iHeight); }

  /* =============================================================== *
   * ------------------------- SET METHODS ------------------------- *
   * =============================================================== */

  /** Method setPlotHeight()
   *  <p> This method will set the Plot Height.
   * @param iScale    = The Time Track Scale identifier
   * @param tsMaximum = The Maximum Date
   * @param tsMinimum = The Minimum Date
   */

  public void setPlotHeight(int iScale, Timestamp tsMaximum, Timestamp tsMinimum)
  {
    this.tsStart = new Timestamp(tsMinimum.getTime());
    this.tsEnd   = new Timestamp(tsMaximum.getTime());
    this.iScale  = iScale;

    long lstart  = tsMinimum.getTime();
    long lend    = tsMaximum.getTime();
    long diff    = lend-lstart;

    iLogHeight   = (int) ((100 * (double) diff) /
                     seismic.seisTracksStruct.SCALE[iScale]);

    iHeight      = PLOT_TITLES + iLogHeight;
  }

  /** Method setData( seisStruct st )
   * <p> This method will set the seismic Data Structure
   * @param st - seismic Data Structure
   */

  public void setLimits(double dMinimum, double dMaximum)
  {
	if (st != null)
	{
	  st.dMinimum = dMinimum;
	  st.dMaximum = dMaximum;
	}
  }

  /** Method setData( seisStruct st )
   * <p> This method will set the seismic Data Structure
   * @param st - seismic Data Structure
   */

  public void setData(seisStruct st) { this.st = st; }

  /** Method setBaro( seisStruct st )
   * <p> This method will set the seismic Data Structure
   * @param st - seismic Data Structure
   */

  public void setBaro(seisStruct stBaro) { this.stBaro = stBaro; }

  /** Method setTidal()
   *  <p> This method will set the Tidal Parameters Data Structure
   *  @param stTidal = the Tidal Parameters Data Structure
   */

  public void setTidal(psiTidalStruct stTidal)
  {
	this.stTidal = stTidal;

    switch (stTidal.iSubtract)
    {
      case _MEAN:
	    stMod = psi.math.psiMathTides.psiArtificial(
			      st,
                  stTidal.dWell, stTidal.dPHI, stTidal.dCw, 5,
                  tsStart, tsEnd,
                  stTidal.dMean, stTidal.dMean,
                  stTidal.dPHId);
        break;
      case _LINEAR:
	    stMod = psi.math.psiMathTides.psiArtificial(
			      st,
                  stTidal.dWell, stTidal.dPHI, stTidal.dCw, 5,
                  tsStart, tsEnd,
                  stTidal.dStart, stTidal.dEnd,
                  stTidal.dPHId );
        break;
	}
  }

  /* =============================================================== *
   * ------------------------ DRAW METHODS ------------------------- *
   * =============================================================== */

  /** Method drawGrid( Graphics g, int iSEIS, int iStartTrack, int iWidthTrack )
   * <p> This method will draw the depth scale
   * @param g           = Graphics Parameter.
   * @param iSEIS        = The LAS Track to draw
   * @param iStartTrack = The starting pixel for the LAS Track
   * @param iWidthTrack = The width of LAS Track in pixels
   */

  public void drawGrid( Graphics g, int iSEIS, int iStartTrack, int iWidthTrack )
  {
    int    i=0;
    int    j=0;
    int    jXinc  = 0;
    int    iLog   = 0;
    int    iCycle = 1;
    double dXinc  = 0.0;
    double dTemp  = 0.0;
    int    iIncrement  = iWidthTrack/4;
    int    iEndTrack   = iStartTrack+iWidthTrack;
//    int    iTrack      = iSEIS - seismic.seisTracksStruct.LAS_START;
    int    iTrack      = iSEIS;
    int    iCurve      = seismic.seisTracksStruct.SEIS_TRACK[iTrack][0];
    int    iValue      = 0;
    int    iY1         = 0;
    int    iY2         = 0;

    double dMinimum = -100.0;
    double dMaximum =  100.0;

    if (st != null)
    {
	  dMaximum = st.dMaximum;
	  dMinimum = st.dMinimum;
	}

    g.setColor(Color.black);

    Font fsb = new Font("Serif", Font.BOLD, 12);
    FontMetrics fsbm = g.getFontMetrics(fsb);
    g.setFont( fsb );

    for (i=0; i<=iLogHeight; i+=iIncrementY)
    {
      iY1 = i+PLOT_TITLES;

//      jXinc = iIncrementY / 5;
//      for (j=0; j<5; j++)
      jXinc = iIncrementY / 6;
      for (j=0; j<6; j++)
      {
        iY2 = i+(j*jXinc)+PLOT_TITLES;

        if ((iY2 >= PLOT_TITLES)  && (iY2 < iHeight))
        {
          g.setColor(Color.lightGray);
          g.drawLine(iStartTrack, iY2, iEndTrack, iY2);
        }
      }

      g.setColor(Color.gray);
      g.drawLine(iStartTrack, iY1, iEndTrack, iY1);
    }

    g.setColor(Color.gray);
    g.drawLine(iEndTrack, LABELSTART, iEndTrack, PLOT_TITLES+iLogHeight);

    for (i=0; i<=iWidthTrack; i+=iIncrement)
    {
      g.setColor(Color.gray);
      if ((i==0) || (i==iWidthTrack))
      {
        g.drawLine(i+iStartTrack, LABELSTART,
                   i+iStartTrack, PLOT_TITLES+iLogHeight);
      }
      else
      {
        g.drawLine(i+iStartTrack, PLOT_TITLES,
                   i+iStartTrack, PLOT_TITLES+iLogHeight);
      }

//        g.setColor(Color.lightGray);
      g.setColor(new Color(230, 230, 230));
      for (j=1; j<5; j++)
      {
        dTemp = (double) iIncrement / 5.0;
        jXinc = i + iStartTrack + j * (int) dTemp;

        if (jXinc <= iEndTrack)
        {
          g.drawLine(jXinc, PLOT_TITLES,
                     jXinc, PLOT_TITLES+iLogHeight);
        }
      }
    }

    g.setColor(Color.black);
    g.drawLine(iStartTrack, LABELSTART,  iEndTrack,   LABELSTART);
    g.drawLine(iStartTrack, PLOT_TITLES, iEndTrack,   PLOT_TITLES);
    g.drawLine(iStartTrack, LABELSTART,  iStartTrack, PLOT_TITLES+iLogHeight); //iHeight);
    g.drawLine(iEndTrack,   LABELSTART,  iEndTrack,   PLOT_TITLES+iLogHeight); //iHeight);
  }

  /** Method drawTrackLabels( Graphics g,
   *                          int iSEIS,
   *                          int iStartTrack,
   *                          int iWidthTrack )
   *  <p> This method will draw the labels on the LAS Plot.
   *   @param g           = Graphics Parameter.
   *   @param iSEIS        = The LAS Track to draw
   *   @param iStartTrack = The starting pixel for the LAS Track
   *   @param iWidthTrack = The width of LAS Track in pixels
   */

  public void drawTrackLabels( Graphics g,
                               int iSEIS,
                               int iStartTrack,
                               int iWidthTrack )
  {
    int    i=0;
    int    iCurve      = -1;
    int    iContinue   = 0;
    int    iLine       = -1;
    int    length      = 0;
    String sLabel      = "";
    String sTemp       = "";
    String sUnits      = "";
    int    iEndTrack   = iStartTrack+iWidthTrack;
    int    iTrack      = iSEIS;

    double dMinimum    = -100.0;
    double dMaximum    =  100.0;

    Font fsb = new Font("Serif", Font.BOLD, 10);
    g.setFont( fsb );

    if (st != null)
    {
      dMaximum = st.dMaximum;
	  dMinimum = st.dMinimum;
	}

    iLine = 3;
    for (i=0; i<5; i++)
    {
      iCurve = seismic.seisTracksStruct.SEIS_TRACK[iTrack][i];
      iContinue = 1;

      if ((iCurve > -1) && (iContinue == 1))
      {
        sLabel = new String(seismic.seisStandardTools.CURVES[iCurve][1]);
        length = sLabel.length();

        if (iLine > -1)
        {
          g.setColor(COLORS[i]);
          g.drawString( sLabel,
                       (iStartTrack+iWidthTrack/2 - 2*length),
                       LABELSTART + (iLine+2)*14);

          g.drawString( "" + dMinimum,
                       iStartTrack+1,
                       LABELSTART + (iLine+2)*14);

          sTemp = new String("" + dMaximum);
          length = sTemp.length();
          g.drawString( "" + dMaximum,
                        iEndTrack-length*5,
                       LABELSTART + (iLine+2)*14);
        }
      }
	}

    if (st != null)
    {
      if ((iSEIS == seismic.seisTracksStruct._SEISMIC) ||
          (iSEIS == seismic.seisTracksStruct._SEISMIC_AVG) ||
          (iSEIS == seismic.seisTracksStruct._SEISMIC_MAG))
      {
        iLine  = 0;
        if (st != null)
          sLabel = new String("Station: "+ st.station+ " Channel: "+st.channel);
        length = sLabel.length();

        g.setColor(Color.black);
        g.drawString( sLabel,
                      (iStartTrack+iWidthTrack/2 - 2*length),
                       LABELSTART + (iLine+2)*14);
	  }
	  else
      {
        iLine  = 0;
        if (st != null)
          sLabel = new String(st.location);
        length = sLabel.length();

        g.setColor(Color.black);
        g.drawString( sLabel,
                      (iStartTrack+iWidthTrack/2 - 2*length),
                       LABELSTART + (iLine+2)*14);
	  }

      if ((iSEIS == seismic.seisTracksStruct._PRESSURE_AVG) ||
          (iSEIS == seismic.seisTracksStruct._SEISMIC_AVG))
      {
        iLine  = 1;
        sLabel = new String(st.sData);
        length = sLabel.length();

        g.setColor(Color.black);
        g.drawString( sLabel,
                      (iStartTrack+iWidthTrack/2 - 2*length),
                       LABELSTART + 2*14 + iLine*11);
//                         LABELSTART + (iLine+2)*14);

        iLine  = 2;
        sLabel = new String(st.sData2);
        length = sLabel.length();

        g.setColor(Color.black);
        g.drawString( sLabel,
                      (iStartTrack+iWidthTrack/2 - 2*length),
                       LABELSTART + 2*14 + iLine*11);
//                         LABELSTART + (iLine+2)*14);

        iLine  = 3;
        sLabel = new String(st.sData3);
        length = sLabel.length();

        g.setColor(Color.black);
        g.drawString( sLabel,
                      (iStartTrack+iWidthTrack/2 - 2*length),
                       LABELSTART + 2*14 + iLine*11);
//                         LABELSTART + (iLine+2)*14);
	  }
	}

    g.setColor(Color.black);
    g.drawLine(iStartTrack, LABELSTART, iEndTrack,   LABELSTART);

    sLabel = new String( seismic.seisTracksStruct.SEIS_TRACK_DESC[iTrack] );
    length = sLabel.length();
    g.drawString(sLabel, iStartTrack+iWidthTrack/2-(9*length)/4, LABELSTART+14);
  }

  /** Method drawTrackCurves( Graphics g,
   *                          int iSEIS,
   *                          int iStartTrack,
   *                          int iWidthTrack )
   *  <p> This method will draw the curves on the LAS Plot.
   *   @param g           = Graphics Parameter.
   *   @param iSEIS        = The LAS Track to draw
   *   @param iStartTrack = The starting pixel for the LAS Track
   *   @param iWidthTrack = The width of LAS Track in pixels
   */

  public void drawTrackCurves( Graphics g,
                               int iSEIS,
                               int iStartTrack,
                               int iWidthTrack )
  {
    int    iColumns    = 0;
    int    iPlot       = 0;
    int    iX1         = 0;
    int    iX2         = 0;
    int    iY1         = 0;
    int    iY2         = 0;
    int    iLog        = 0;
    int    iColor      = 0;
    double dRatio      = 0.0;
    String sUnits      = "";
    double dLog[]      = null;
    double dStart      = 0.0;
    double dEnd        = 0.0;
    double data        = 0.0;
    double data1       = 0.0;
    int    iEndTrack   = iStartTrack+iWidthTrack;
//    int    iTrack      = iSEIS - seismic.seisTracksStruct.LAS_START;
    int    iTrack      = iSEIS;
    int    iCurve      = seismic.seisTracksStruct.SEIS_TRACK[iTrack][0];

    double dMinimum = -100.0;
    double dMaximum =  100.0;

    if (st != null)
    {
	  dMaximum = st.dMaximum;
	  dMinimum = st.dMinimum;
	}

    if (st != null)
    {
      for (int i=0; i<5; i++)
      {
        dLog   = null;
        iCurve = seismic.seisTracksStruct.SEIS_TRACK[iTrack][i];

        if (iCurve > -1)
        {
          iColor   = i;
          iColumns = st.iRows-1;

          dLog = st.data; //st.getData(iCurve);

          if (dLog != null)
          {
            for (int j=0; j<iColumns-1; j++)
            {
              if (st != null)
              {
                long lstart = tsStart.getTime();
                long lend   = tsEnd.getTime();

				long l1     = st.ts[j].getTime();
				long l2     = st.ts[j+1].getTime();

                data        = dLog[j];
                data1       = dLog[j+1];

                dRatio = (double) iLogHeight * (l1 - lstart) / (lend - lstart);
                iY1 = PLOT_TITLES + (int) dRatio;

                dRatio = (double) iLogHeight * (l2 - lstart) / (lend - lstart);
                iY2 = PLOT_TITLES + (int) dRatio;

                iPlot = 0;
                if ((iY1 > PLOT_TITLES) && (iY2 > PLOT_TITLES) &&
                    (iY1 < iHeight)     && (iY2 < iHeight))
                {
                  iPlot = 1;
                }

                dRatio = (double) iWidthTrack *
                      (data - dMinimum) / (dMaximum - dMinimum);
                iX1 = iStartTrack + (int) dRatio;

                dRatio = (double) iWidthTrack *
                      (data1 - dMinimum) / (dMaximum - dMinimum);
                iX2 = iStartTrack + (int) dRatio;

                if ((iX1 >= iStartTrack) && (iX2 >= iStartTrack) &&
                    (iX1 <= iEndTrack)   && (iX2 <= iEndTrack))
                {
                  g.setColor(COLORS[iColor]);
                  if (iPlot == 1)
                    g.drawLine(iX1, iY1, iX2, iY2);
			    }
			  }
			}
		  }
		}
	  }
	}
  }

  /** Method drawArtificialPressure()
   *  <p> This method will draw the artificial pressure
   *      on top of the average pressure
   *   @param g           = Graphics Parameter.
   *   @param iSEIS        = The LAS Track to draw
   *   @param iStartTrack = The starting pixel for the LAS Track
   *   @param iWidthTrack = The width of LAS Track in pixels
   */

  public void drawArtificialPressure( Graphics g,
                                      int iSEIS,
                                      int iStartTrack,
                                      int iWidthTrack )
  {
    int    iColumns    = 0;
    int    iPlot       = 0;
    int    iXo         = 0;
    int    iX1         = 0;
    int    iX2         = 0;
    int    iY1         = 0;
    int    iY2         = 0;
    int    iX1m        = 0;
    int    iX2m        = 0;
    int    iColor      = 0;
    double dRatio      = 0.0;
    double dMinimum    = -100.0;
    double dMaximum    =  100.0;

    int    iEndTrack   = iStartTrack+iWidthTrack;

    long   lstart      = (long) 0;
    long   lend        = (long) 0;
    long   l1          = (long) 0;
    long   l2          = (long) 0;

    long   lstartS     = (long) 0;
    long   lendS       = (long) 0;

    double dPStart     = 0.0;
    double dPEnd       = 0.0;

	double data1       = 0.0;
	double data2       = 0.0;
	double dMean1      = 0.0;
	double dMean2      = 0.0;
	double dTime       = 0.0;
	double dRange      = 0.0;
	double dpsi        = 0.0;

    if (iSEIS == seismic.seisTracksStruct._PRESSURE_AVG)
    {
      lstart = tsStart.getTime();
      lend   = tsEnd.getTime();

      lstartS = st.tsStart.getTime();
      lendS   = st.tsEnd.getTime();

      if ((st != null) && (stTidal != null))
      {
		switch (stTidal.iSubtract)
		{
		  case _MEAN:
		    dPStart = stTidal.dMean;
		    dPEnd   = stTidal.dMean;
		    break;

		  case _LINEAR:
		    dPStart = stTidal.dStart;
		    dPEnd   = stTidal.dEnd;
		    break;
		}

	    dMaximum = st.dMaximum;
	    dMinimum = st.dMinimum;
	    iColumns = st.iRows-1;

        for (int j=0; j<iColumns-1; j++)
        {
	  	  l1     = st.ts[j].getTime();
		  l2     = st.ts[j+1].getTime();

          dRatio = (double) ((iLogHeight * (l1 - lstart)) / (lend - lstart));
          iY1 = PLOT_TITLES + (int) dRatio;

          dRatio = (double) ((iLogHeight * (l2 - lstart)) / (lend - lstart));
          iY2 = PLOT_TITLES + (int) dRatio;

          iPlot = 0;
          if ((iY1 > PLOT_TITLES) && (iY2 > PLOT_TITLES) &&
              (iY1 < iHeight)     && (iY2 < iHeight))
          {
            iPlot = 1;
          }

//		  dRange = (double) (lend - lstart) / 1000.0;
//		  dTime  = (double) (lend - l1) / 1000.0;
		  dRange = (double) (lendS - lstartS) / 1000.0;
		  dTime  = (double) (lendS - l1) / 1000.0;
		  dMean1 = dPEnd - (dPEnd-dPStart) * dTime / dRange;
		  data1 = stMod.data[j];

//		  dTime  = (double) (lend - l2) / 1000.0;
		  dTime  = (double) (lendS - l2) / 1000.0;
		  dMean2 = dPEnd - (dPEnd-dPStart) * dTime / dRange;
		  data2 = stMod.data[j+1];

          dRatio = (double) iWidthTrack *
                   (data1 - dMinimum) / (dMaximum - dMinimum);
          iX1 = iStartTrack + (int) dRatio;

          dRatio = (double) iWidthTrack *
                   (data2 - dMinimum) / (dMaximum - dMinimum);
          iX2 = iStartTrack + (int) dRatio;

          dRatio = (double) iWidthTrack *
                   (dMean1 - dMinimum) / (dMaximum - dMinimum);
          iX1m = iStartTrack + (int) dRatio;

          dRatio = (double) iWidthTrack *
                   (dMean2 - dMinimum) / (dMaximum - dMinimum);
          iX2m = iStartTrack + (int) dRatio;

          if ((iX1 >= iStartTrack) && (iX2 >= iStartTrack) &&
              (iX1 <= iEndTrack)   && (iX2 <= iEndTrack))
          {
            g.setColor(COLORS[2]);
            if (iPlot == 1)
              g.drawLine(iX1, iY1, iX2, iY2);
		  }

          if ((iX1m >= iStartTrack) && (iX2m >= iStartTrack) &&
              (iX1m <= iEndTrack)   && (iX2m <= iEndTrack))
          {
            g.setColor(COLORS[0]);
            if (iPlot == 1)
              g.drawLine(iX1m, iY1, iX2m, iY2);
		  }
		}
	  }
	}
  }

  /** Method drawSlopePressure()
   *  <p> This method will draw the artificial pressure
   *      on top of the average pressure
   *   @param g           = Graphics Parameter.
   *   @param iSEIS        = The LAS Track to draw
   *   @param iStartTrack = The starting pixel for the LAS Track
   *   @param iWidthTrack = The width of LAS Track in pixels
   */

  public void drawSlopePressure( Graphics g,
                                 int iSEIS,
                                 int iStartTrack,
                                 int iWidthTrack )
  {
    int    iColumns    = 0;
    int    iPlot       = 0;
    int    iXo         = 0;
    int    iX1         = 0;
    int    iX2         = 0;
    int    iY1         = 0;
    int    iY2         = 0;
    int    iX1m        = 0;
    int    iX2m        = 0;
    int    iColor      = 0;
    double dRatio      = 0.0;
    double dMinimum    = -100.0;
    double dMaximum    =  100.0;

    int    iEndTrack   = iStartTrack+iWidthTrack;

    long   lstart      = (long) 0;
    long   lend        = (long) 0;
    long   l1          = (long) 0;
    long   l2          = (long) 0;

    long   lstartS     = (long) 0;
    long   lendS       = (long) 0;

    double dPStart     = 0.0;
    double dPEnd       = 0.0;

	double data1       = 0.0;
	double data2       = 0.0;
	double dMean1      = 0.0;
	double dMean2      = 0.0;
	double dTime       = 0.0;
	double dRange      = 0.0;
	double dpsi        = 0.0;

    if ((iSEIS == seismic.seisTracksStruct._PRESSURE_MOD) && (st != null))
    {
      lstart = tsStart.getTime();
      lend   = tsEnd.getTime();

      lstartS = st.tsStart.getTime();
      lendS   = st.tsEnd.getTime();

      if ((st != null) && (stTidal != null))
      {
		switch (stTidal.iSubtract)
		{
		  case _MEAN:
		    dPStart = stTidal.dMean;
		    dPEnd   = stTidal.dMean;
		    break;

		  case _LINEAR:
		    dPStart = stTidal.dStart;
		    dPEnd   = stTidal.dEnd;
		    break;
		}

	    dMaximum = st.dMaximum;
	    dMinimum = st.dMinimum;
	    iColumns = st.iRows-1;

        for (int j=0; j<iColumns-1; j++)
        {
	  	  l1     = st.ts[j].getTime();
		  l2     = st.ts[j+1].getTime();

          dRatio = (double) ((iLogHeight * (l1 - lstart)) / (lend - lstart));
          iY1 = PLOT_TITLES + (int) dRatio;

          dRatio = (double) ((iLogHeight * (l2 - lstart)) / (lend - lstart));
          iY2 = PLOT_TITLES + (int) dRatio;

          iPlot = 0;
          if ((iY1 > PLOT_TITLES) && (iY2 > PLOT_TITLES) &&
              (iY1 < iHeight)     && (iY2 < iHeight))
          {
            iPlot = 1;
          }

//		  dRange = (double) (lend - lstart) / 1000.0;
//		  dTime  = (double) (lend - l1) / 1000.0;
		  dRange = (double) (lendS - lstartS) / 1000.0;
		  dTime  = (double) (lendS - l1) / 1000.0;
		  dMean1 = dPEnd - (dPEnd-dPStart) * dTime / dRange;
//		  data1 = stMod.data[j];

//		  dTime  = (double) (lend - l2) / 1000.0;
		  dTime  = (double) (lendS - l2) / 1000.0;
		  dMean2 = dPEnd - (dPEnd-dPStart) * dTime / dRange;
//		  data2 = stMod.data[j+1];
/*
          dRatio = (double) iWidthTrack *
                   (data1 - dMinimum) / (dMaximum - dMinimum);
          iX1 = iStartTrack + (int) dRatio;

          dRatio = (double) iWidthTrack *
                   (data2 - dMinimum) / (dMaximum - dMinimum);
          iX2 = iStartTrack + (int) dRatio;
*/
          dRatio = (double) iWidthTrack *
                   (dMean1 - dMinimum) / (dMaximum - dMinimum);
          iX1m = iStartTrack + (int) dRatio;

          dRatio = (double) iWidthTrack *
                   (dMean2 - dMinimum) / (dMaximum - dMinimum);
          iX2m = iStartTrack + (int) dRatio;
/*
          if ((iX1 >= iStartTrack) && (iX2 >= iStartTrack) &&
              (iX1 <= iEndTrack)   && (iX2 <= iEndTrack))
          {
            g.setColor(COLORS[2]);
            if (iPlot == 1)
              g.drawLine(iX1, iY1, iX2, iY2);
		  }
*/
          if ((iX1m >= iStartTrack) && (iX2m >= iStartTrack) &&
              (iX1m <= iEndTrack)   && (iX2m <= iEndTrack))
          {
            g.setColor(COLORS[0]);
            if (iPlot == 1)
              g.drawLine(iX1m, iY1, iX2m, iY2);
		  }
		}
	  }
	}
  }

  /** Method drawBarometric()
   *  <p> This method will draw the artificial pressure
   *      on top of the average pressure
   *   @param g           = Graphics Parameter.
   *   @param iSEIS        = The LAS Track to draw
   *   @param iStartTrack = The starting pixel for the LAS Track
   *   @param iWidthTrack = The width of LAS Track in pixels
   */

  public void drawBarometric( Graphics g,
                              int iSEIS,
                              int iStartTrack,
                              int iWidthTrack )
  {
    int    iColumns    = 0;
    int    iPlot       = 0;
    int    iXo         = 0;
    int    iX1         = 0;
    int    iX2         = 0;
    int    iY1         = 0;
    int    iY2         = 0;
    int    iX1m        = 0;
    int    iX2m        = 0;
    int    iColor      = 0;
    double dRatio      = 0.0;
    double dMinimum    = -100.0;
    double dMaximum    =  100.0;

    int    iEndTrack   = iStartTrack+iWidthTrack;

    long   lstart      = (long) 0;
    long   lend        = (long) 0;
    long   l1          = (long) 0;
    long   l2          = (long) 0;

    long   lstartS     = (long) 0;
    long   lendS       = (long) 0;

    double dPStart     = 0.0;
    double dPEnd       = 0.0;

	double data1       = 0.0;
	double data2       = 0.0;
	double dMean1      = 0.0;
	double dMean2      = 0.0;
	double dTime       = 0.0;
	double dRange      = 0.0;
	double dpsi        = 0.0;

    double dColumn     = 0.0;

    if ((iSEIS == seismic.seisTracksStruct._PRESSURE_MOD) &&
        (stBaro != null) && (stMod != null))
    {
      lstart = tsStart.getTime();
      lend   = tsEnd.getTime();

      lstartS = st.tsStart.getTime();
      lendS   = st.tsEnd.getTime();

      if ((st != null) && (stTidal != null))
      {
        dColumn = stTidal.dPHIs;

		switch (stTidal.iSubtract)
		{
		  case _MEAN:
		    dPStart = stTidal.dMean;
		    dPEnd   = stTidal.dMean;
		    break;

		  case _LINEAR:
		    dPStart = stTidal.dStart;
		    dPEnd   = stTidal.dEnd;
		    break;
		}

	    dMaximum = st.dMaximum;
	    dMinimum = st.dMinimum;

	    iColumns = stBaro.iRows-1;

        for (int j=0; j<iColumns-1; j++)
        {
	  	  l1     = stBaro.ts[j].getTime();
		  l2     = stBaro.ts[j+1].getTime();

          dRatio = (double) ((iLogHeight * (l1 - lstart)) / (lend - lstart));
          iY1 = PLOT_TITLES + (int) dRatio;

          dRatio = (double) ((iLogHeight * (l2 - lstart)) / (lend - lstart));
          iY2 = PLOT_TITLES + (int) dRatio;

          iPlot = 0;
          if ((iY1 > PLOT_TITLES) && (iY2 > PLOT_TITLES) &&
              (iY1 < iHeight)     && (iY2 < iHeight))
          {
            iPlot = 1;
          }


		  dRange = (double) (lendS - lstartS) / 1000.0;
		  dTime  = (double) (lendS - l1) / 1000.0;
		  dMean1 = dPEnd - (dPEnd-dPStart) * dTime / dRange;
		  dMean1 = dMean1 - stTidal.dPHIs;

		  dTime  = (double) (lendS - l2) / 1000.0;
		  dMean2 = dPEnd - (dPEnd-dPStart) * dTime / dRange;
          dMean2 = dMean2 - stTidal.dPHIs;

          dMean1 = stBaro.data[j] + dMean1;
          dRatio = (double) iWidthTrack *
                   (dMean1 - dMinimum) / (dMaximum - dMinimum);
          iX1m = iStartTrack + (int) dRatio;

          dMean2 = stBaro.data[j+1] + dMean2;
          dRatio = (double) iWidthTrack *
                   (dMean2 - dMinimum) / (dMaximum - dMinimum);
          iX2m = iStartTrack + (int) dRatio;

          if ((iX1m >= iStartTrack) && (iX2m >= iStartTrack) &&
              (iX1m <= iEndTrack)   && (iX2m <= iEndTrack))
          {
            g.setColor(COLORS[7]);
            if (iPlot == 1)
              g.drawLine(iX1m, iY1, iX2m, iY2);
		  }
		}
	  }
	}
  }

  /** Method drawLunarTide( Graphics g,
   *                          int iSEIS,
   *                          int iStartTrack,
   *                          int iWidthTrack )
   *  <p> This method will draw the curves on the LAS Plot.
   *   @param g           = Graphics Parameter.
   *   @param iSEIS        = The LAS Track to draw
   *   @param iStartTrack = The starting pixel for the LAS Track
   *   @param iWidthTrack = The width of LAS Track in pixels
   */

  public void drawLunarTide( Graphics g,
                             int iSEIS,
                             int iStartTrack,
                             int iWidthTrack )
  {
    int    iColumns    = 0;
    int    iPlot       = 0;
    int    iXo         = 0;
    int    iX1         = 0;
    int    iX2         = 0;
    int    iY1         = 0;
    int    iY2         = 0;
    int    iLog        = 0;
    int    iColor      = 0;
    double dRatio      = 0.0;
    String sUnits      = "";
    double dLog[]      = null;
    double dStart      = 0.0;
    double dEnd        = 0.0;
    double data        = 0.0;
    double data1       = 0.0;
    int    iEndTrack   = iStartTrack+iWidthTrack;
//    int    iTrack      = iSEIS - seismic.seisTracksStruct.LAS_START;
    int    iTrack      = iSEIS;
    int    iCurve      = seismic.seisTracksStruct.SEIS_TRACK[iTrack][0];
    double del         = 0.0;
    long   lp          = (long) 10800000;
    long   lt          = (long) 0;
    double F           = (2.0*Math.PI) * 0.000022182;
    double delta       = 0.19433 * Math.PI;
    double P           = 0.04;
    double F2          = (2.0*Math.PI) * 0.0000118032;
    double delta2      = 0.90472 * Math.PI;
    double P2          = 0.027;

    double dMinimum = -100.0;
    double dMaximum =  100.0;

    if (iTrack == seismic.seisTracksStruct._PRESSURE_MOD)
    {
      if (st != null)
      {
	    dMaximum = st.dMaximum;
	    dMinimum = st.dMinimum;
	  }

      if (st != null)
      {
        for (int i=0; i<5; i++)
        {
          dLog   = null;
          iCurve = seismic.seisTracksStruct.SEIS_TRACK[iTrack][i];

          if (iCurve > -1)
          {
            iColor   = i;
            iColumns = st.iRows-1;

            dLog = st.data; //st.getData(iCurve);

            if (dLog != null)
            {
              for (int j=0; j<iColumns-1; j++)
              {
                if (st != null)
                {
                  long lstart = tsStart.getTime();
                  long lend   = tsEnd.getTime();

	  			  long l1     = st.ts[j].getTime();
				  long l2     = st.ts[j+1].getTime();

                  lt    = lstart;// + lp;
//                  F     = (2.0*Math.PI) * 0.00002315;
//                  delta = 0.519*Math.PI;

                  del   = (double) (l1-lt) / 1000.0;
                  data  = P * Math.cos(F*del + delta) +
                          P2 * Math.cos(F2*del + delta2);

                  del   = (double) (l2-lt) / 1000.0;
                  data1 = P * Math.cos(F*del + delta) +
                          P2 * Math.cos(F2*del + delta2);

//                  del   = (double) (l1) / 1000.0;
//                  del   = j*60.0;
//                  data  = P * Math.cos(F*del - delta) +
//                          P2 * Math.cos(F2*del - delta2);

//                  del   = (double) (l2) / 1000.0;
//                  del   = (j+1)*60.0;
//                  data1 = P * Math.cos(F*del - delta) +
//                          P2 * Math.cos(F2*del - delta2);
//                  data        = dLog[j];
//                  data1       = dLog[j+1];

                  dRatio = (double) iLogHeight * (l1 - lstart) / (lend - lstart);
                  iY1 = PLOT_TITLES + (int) dRatio;

                  dRatio = (double) iLogHeight * (l2 - lstart) / (lend - lstart);
                  iY2 = PLOT_TITLES + (int) dRatio;

                  iPlot = 0;
                  if ((iY1 > PLOT_TITLES) && (iY2 > PLOT_TITLES) &&
                      (iY1 < iHeight)     && (iY2 < iHeight))
                  {
                    iPlot = 1;
                  }

                  dRatio = (double) iWidthTrack *
                        (data - dMinimum) / (dMaximum - dMinimum);
                  iX1 = iStartTrack + (int) dRatio;

                  dRatio = (double) iWidthTrack *
                        (data1 - dMinimum) / (dMaximum - dMinimum);
                  iX2 = iStartTrack + (int) dRatio;

                  dRatio = (double) iWidthTrack *
                        (0.0 - dMinimum) / (dMaximum - dMinimum);
                  iXo = iStartTrack + (int) dRatio;

                  if ((iX1 >= iStartTrack) && (iX2 >= iStartTrack) &&
                      (iX1 <= iEndTrack)   && (iX2 <= iEndTrack))
                  {
                    g.setColor(COLORS[iColor]);
                    if (iPlot == 1)
                      g.drawLine(iX1, iY1, iX2, iY2);

                    g.setColor(COLORS[0]);
                    if (iPlot == 1)
                      g.drawLine(iXo, iY1, iXo, iY2);
			      }
			    }
			  }
			}
		  }
		}
	  }
	}
  }

  /** Method draw( Graphics g, int iSEIS, int iStartTrack, int iWidthTrack )
   * <p> This method will create a LAS Plot Track
   * @param g           = Graphics Parameter.
   * @param iSEIS        = The LAS Track to draw
   * @param iStartTrack = The starting pixel for the LAS Track
   * @param iWidthTrack = The width of LAS Track in pixels
   */

  public void draw( Graphics g, int iSEIS, int iStartTrack, int iWidthTrack )
  {
    int iColor = seismic.seisTracksStruct._SRC_SEISMIC;

    if ((iSEIS == seismic.seisTracksStruct._SEISMIC) ||
        (iSEIS == seismic.seisTracksStruct._SEISMIC_AVG) ||
        (iSEIS == seismic.seisTracksStruct._SEISMIC_MAG) ||
        (iSEIS == seismic.seisTracksStruct._PRESSURE_TEST) ||
        (iSEIS == seismic.seisTracksStruct._PRESSURE_AVG) ||
        (iSEIS == seismic.seisTracksStruct._PRESSURE_MOD) ||
        (iSEIS == seismic.seisTracksStruct._PRESSURE_BARO))
    {
	  if ((iSEIS == seismic.seisTracksStruct._PRESSURE_TEST) ||
          (iSEIS == seismic.seisTracksStruct._PRESSURE_AVG) ||
          (iSEIS == seismic.seisTracksStruct._PRESSURE_MOD) ||
          (iSEIS == seismic.seisTracksStruct._PRESSURE_BARO))
        iColor = seismic.seisTracksStruct._SRC_PSI;

      g.setColor( new Color( seismic.seisTracksStruct.COLORS[iColor][0],
                             seismic.seisTracksStruct.COLORS[iColor][1],
                             seismic.seisTracksStruct.COLORS[iColor][2] ) );
      g.fillRect( iStartTrack, LABELSTART, iWidthTrack, 20 );

      drawGrid( g, iSEIS, iStartTrack, iWidthTrack );
      drawTrackLabels( g, iSEIS, iStartTrack, iWidthTrack );
      drawTrackCurves( g, iSEIS, iStartTrack, iWidthTrack );

      switch (iSEIS)
      {
		case seismic.seisTracksStruct._PRESSURE_AVG:
          drawArtificialPressure( g, iSEIS, iStartTrack, iWidthTrack );
          break;

        case seismic.seisTracksStruct._PRESSURE_MOD:
          drawSlopePressure( g, iSEIS, iStartTrack, iWidthTrack );
          drawBarometric( g, iSEIS, iStartTrack, iWidthTrack );
          break;
	  }
//      drawLunarTide( g, iSEIS, iStartTrack, iWidthTrack );
	}
  }

  /** Method paint( Graphics g )
   * <p> This method will paint Porosity & Permeability Plot
   * @param g = Graphics Parameter.
   */

  public void paint( Graphics g )
  {
    g.setColor(Color.white);
    g.fillRect(0, 0, iWidth, iHeight);
  }
}

/*
 *  @version 1.1 03/04/2015
 *  @author  John Victorine
 */
